@@ -1,5 +1,5 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 """this file contains the version of translate"""
-ver = "0.10.1"
-build = "9900"
+ver = "1.0.1"
+build = 10000
