@@ -34,9 +34,10 @@ except ImportError:
 
 class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
   """a specialized Option Parser for convertor tools..."""
-  def __init__(self, formats, usetemplates=False, usepots=False, description=None):
+  def __init__(self, formats, usetemplates=False, usepots=False, allowmissingtemplate=False, description=None):
     """construct the specialized Option Parser"""
-    optrecurse.RecursiveOptionParser.__init__(self, formats, usetemplates, description=description)
+    optrecurse.RecursiveOptionParser.__init__(self, formats, usetemplates, 
+            allowmissingtemplate=allowmissingtemplate, description=description)
     self.usepots = usepots
     self.setpotoption()
     self.set_usage()
@@ -56,8 +57,8 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
   def add_duplicates_option(self, default="merge"):
     """adds an option to say what to do with duplicate strings"""
     self.add_option("", "--duplicates", dest="duplicatestyle", default=default,
-      type="choice", choices=["msgid_comment", "merge", "keep", "msgid_comment_all"],
-      help="what to do with duplicate strings (identical source text): merge (default), msgid_comment, keep, msgid_comment_all", metavar="DUPLICATESTYLE")
+      type="choice", choices=["msgid_comment", "msgctxt", "merge", "keep", "msgid_comment_all"],
+      help="what to do with duplicate strings (identical source text): merge (default), msgid_comment, msgctxt, keep, msgid_comment_all", metavar="DUPLICATESTYLE")
     self.passthrough.append("duplicatestyle")
 
   def add_multifile_option(self, default="single"):
