@@ -128,3 +128,6 @@ def main(argv=None):
   parser = convert.ConvertOptionParser(formats, usepots=True, description=__doc__)
   parser.run(argv)
 
+
+if __name__ == '__main__':
+    main()
