@@ -38,6 +38,10 @@ class oo2po:
     self.blankmsgstr = blankmsgstr
     self.long_keys = long_keys
 
+  def escape_text(self, text):
+    """Escapes sdf text to be suitable for po consumption."""
+    return text.replace("\\\\", "\a").replace("\\n", "\n").replace("\\t", "\t").replace("\\r", "\r").replace("\a", "\\\\")
+
   def makepo(self, part1, part2, translators_comment, key, subkey):
     """makes a po element out of a subkey of two parts"""
     thepo = po.pounit(encoding="UTF-8")
@@ -45,10 +49,10 @@ class oo2po:
     if getattr(translators_comment, subkey).strip() != "":
       thepo.automaticcomments.append("#. %s\n" % getattr(translators_comment, subkey))
     #TODO: Do better
-    text1 = getattr(part1, subkey).replace("\\\\", "\a").replace("\\n", "\n").replace("\\t", "\t").replace("\\r", "\r").replace("\a", "\\")
-    text2 = getattr(part2, subkey).replace("\\\\", "\a").replace("\\n", "\n").replace("\\t", "\t").replace("\\r", "\r").replace("\a", "\\")
-    thepo.source = text1
-    thepo.target = text2
+    text1 = self.escape_text(getattr(part1, subkey))
+    text2 = self.escape_text(getattr(part2, subkey))
+    thepo.source = text1.decode('utf-8')
+    thepo.target = text2.decode('utf-8')
     return thepo
 
   def makekey(self, ookey):
@@ -96,7 +100,7 @@ class oo2po:
     polist = [textpo, quickhelppo, titlepo]
     return polist
 
-  def convertfile(self, theoofile, duplicatestyle="msgid_comment"):
+  def convertfile(self, theoofile, duplicatestyle="msgctxt"):
     """converts an entire oo file to .po format"""
     thepofile = po.pofile()
     # create a header for the file
@@ -163,3 +167,6 @@ def main(argv=None):
   parser.verifyoptions = verifyoptions
   parser.run(argv)
 
+
+if __name__ == '__main__':
+    main()
