@@ -21,7 +21,7 @@
 #
 
 """Converts Gettext .po files to Qt .ts localization files
-You can convert from .ts to .po using po2ts"""
+You can convert from .ts to .po using ts2po"""
 
 from translate.storage import po
 from translate.storage import ts
@@ -79,3 +79,6 @@ def main(argv=None):
   parser = convert.ConvertOptionParser(formats, usepots=True, usetemplates=True, description=__doc__)
   parser.run(argv)
 
+
+if __name__ == '__main__':
+    main()
