@@ -99,14 +99,15 @@ class TestDTD2PO:
 
     def test_kdecomment_merge(self):
         """test that LOCALIZATION NOTES are added properly as KDE comments and merged with duplicate comments"""
-        dtdtemplate = '<!--LOCALIZATION NOTE (%s): Edit box appears beside this label -->\n' + \
-            '<!ENTITY %s "If publishing to a FTP site, enter the HTTP address to browse to:">\n'
+        dtdtemplate = '<!--LOCALIZATION NOTE (%s): Some note -->\n' + \
+            '<!ENTITY %s "Source text">\n'
         dtdsource = dtdtemplate % ("note1.label", "note1.label") + dtdtemplate % ("note2.label", "note2.label")
         pofile = self.dtd2po(dtdsource)
         pofile.units = pofile.units[1:]
         posource = str(pofile)
         print posource
-        assert posource.count('"_:') <= len(pofile.units)
+        assert posource.count('"_:') == 2
+        assert posource.count('\\n') == 2
 
     def test_donttranslate_simple(self):
         """check that we handle DONT_TRANSLATE messages properly"""
@@ -257,7 +258,7 @@ class TestDTD2PO:
         assert pofile.units[4].source == "M"
         assert pofile.units[4].target == "ﺩ"
 
-    def xtest_accelerator_keys_not_in_sentence(self):
+    def wtest_accelerator_keys_not_in_sentence(self):
         """tests to ensure that we can manage accelerator keys that are not part of the transated sentence eg in Chinese"""
         dtdtemplate = '''<!ENTITY useAutoScroll.label             "Use autoscrolling">
 <!ENTITY useAutoScroll.accesskey         "a">'''
