@@ -12,11 +12,11 @@ class TestPO2XLIFF:
 
     def po2xliff(self, posource, sourcelanguage='en', targetlanguage=None):
         """helper that converts po source to xliff source without requiring files"""
-	pofile = po.pofile(posource)
-	convertor = po2xliff.po2xliff()
-	outputxliff = convertor.convertfile(pofile, None, sourcelanguage=sourcelanguage)
-	return poxliff.PoXliffFile(outputxliff)
-	
+        pofile = po.pofile(posource)
+        convertor = po2xliff.po2xliff()
+        outputxliff = convertor.convertfile(pofile, None, sourcelanguage=sourcelanguage, targetlanguage=targetlanguage)
+        return poxliff.PoXliffFile(outputxliff)
+
     def getnode(self, xliff):
         """Retrieves the trans-unit node from the dom"""
         assert len(xliff.units) == 1
@@ -58,10 +58,10 @@ msgstr "Toepassings"
         assert xliff.translate("Applications") == "Toepassings"
         assert xliff.translate("bla") is None
         xmltext = str(xliff)
-	assert xmltext.index('<xliff version="1.1"')
-	assert xmltext.index('<file')
-	assert xmltext.index('source-language')
-	assert xmltext.index('datatype')
+        assert xmltext.index('<xliff version="1.1"')
+        assert xmltext.index('<file')
+        assert xmltext.index('source-language')
+        assert xmltext.index('datatype')
 
     def test_multiline(self):
         """Test multiline po entry"""
@@ -82,7 +82,7 @@ msgstr "Eerste lyn\nTweede lyn"
 '''
         xliff = self.po2xliff(minipo)
         print "The generated xml:"
-	xmltext = str(xliff)
+        xmltext = str(xliff)
         print xmltext
         assert xliff.translate("First line\nSecond line") == "Eerste lyn\nTweede lyn"
         assert xliff.translate("First line\\nSecond line") is None
@@ -90,7 +90,7 @@ msgstr "Eerste lyn\nTweede lyn"
         assert xmltext.find("lyn\\nTweede") == -1
         assert xmltext.find("line\nSecond") > 0
         assert xmltext.find("lyn\nTweede") > 0
-	
+
     def test_escapedtabs(self):
         """Test the escaping of tabs"""
         minipo = r'''msgid "First column\tSecond column"
@@ -171,7 +171,7 @@ msgstr "kunye"
         tuples = self.getcontexttuples(node)
         assert tuples == [("x-po-trancomment", "Translate?\nHow?")]
 
-        assert xliff.units[0].getnotes() == "Translate?\nHow?"
+        assert xliff.units[0].getnotes("translator") == "Translate?\nHow?"
 
 
     def test_automaticcomments(self):
@@ -213,7 +213,7 @@ msgstr ""
         assert unit.xmlelement.getAttribute("restype") == "x-gettext-domain-header"
         assert unit.xmlelement.getAttribute("approved") == "no"
         assert unit.xmlelement.getAttribute("xml:space") == "preserve"
-        assert unit.getnotes() == "Pulana  Translation for bla\nHallo Ma!"
+        assert unit.getnotes("po-translator") == "Pulana  Translation for bla\nHallo Ma!"
 
     def test_fuzzy(self):
         minipo = r'''#, fuzzy
@@ -241,7 +241,7 @@ msgstr[1] "iinkomo"
         print "The generated xml:"
         xmltext = str(xliff)
         print xmltext
-	assert len(xliff.units) == 1
+        assert len(xliff.units) == 1
         assert xliff.translate("cow") == "inkomo"
         
     def test_funny_plurals(self):
@@ -255,7 +255,7 @@ msgstr[2] "iiinkomo"
         print "The generated xml:"
         xmltext = str(xliff)
         print xmltext
-	assert len(xliff.units) == 1
+        assert len(xliff.units) == 1
         assert xliff.translate("cow") == "inkomo"
 
     def test_language_tags(self):
@@ -266,3 +266,10 @@ msgstr "Uno"
         assert xliff.sourcelanguage == "af"
         assert xliff.targetlanguage == "es"
         
+    def test_variables(self):
+        minipo = r'''msgid "%s%s%s%s has made %s his or her buddy%s%s"
+msgstr "%s%s%s%s het %s sy/haar vriend/vriendin gemaak%s%s"'''
+        xliff = self.po2xliff(minipo)
+        print xliff.units[0].source
+        assert xliff.units[0].source == "%s%s%s%s has made %s his or her buddy%s%s"
+
