@@ -2,6 +2,7 @@
 
 from translate.convert import xliff2po
 from translate.storage import po
+from translate.misc import wStringIO
 
 class TestXLIFF2PO:
     xliffskeleton = '''<?xml version="1.0" ?>
@@ -15,8 +16,9 @@ class TestXLIFF2PO:
 
     def xliff2po(self, xliffsource):
         """helper that converts xliff source to po source without requiring files"""
+        inputfile = wStringIO.StringIO(xliffsource)
 	convertor = xliff2po.xliff2po()
-	outputpo = convertor.convertfile(xliffsource)
+	outputpo = convertor.convertfile(inputfile)
         print "The generated po:"
         print str(outputpo)
         return outputpo
@@ -70,11 +72,14 @@ Content-Transfer-Encoding: 8bit'''
             <context context-type="x-po-trancomment">Couldn't do
 it</context>
         </context-group>
+        <note from="po-translator">Couldn't do
+it</note>
 </trans-unit>'''
         pofile = self.xliff2po(minixlf)
         assert pofile.translate("nonsense") == "matlhapolosa"
         assert pofile.translate("bla") is None
         unit = pofile.units[0]
+        assert unit.getnotes("translator") == "Couldn't do\nit"
         assert unit.othercomments == ["# Couldn't do\n", "# it\n"]
         potext = str(pofile)
         assert potext.index("# Couldn't do\n# it\n") >= 0
@@ -88,11 +93,14 @@ it</context>
             <context context-type="x-po-autocomment">Note that this is
 garbage</context>
         </context-group>
+        <note from="developer">Note that this is
+garbage</note>
 </trans-unit>'''
         pofile = self.xliff2po(minixlf)
         assert pofile.translate("nonsense") == "matlhapolosa"
         assert pofile.translate("bla") is None
         unit = pofile.units[0]
+        assert unit.getnotes("developer") == "Note that this is\ngarbage"
         assert unit.automaticcomments == ["#. Note that this is\n", "#. garbage\n"]
         potext = str(pofile)
         assert potext.index("#. Note that this is\n#. garbage\n") >= 0
