@@ -24,8 +24,8 @@
 You can convert back to .xliff using po2xliff"""
 
 from translate.storage import po
-from translate.storage import poxliff as xliff
-from translate.misc import quote
+from translate.storage import xliff
+from translate.misc import wStringIO
 
 class xliff2po:
   def converttransunit(self, transunit):
@@ -42,38 +42,35 @@ class xliff2po:
     if locations:
       thepo.sourcecomments.append("#: %s\n" % " ".join(locations))
 
+    #NOTE: Supporting both <context> and <note> tags in xliff files for comments
     #Translator comments
-    notes = transunit.getnotes()
-    trancomments = transunit.gettranslatorcomments()
-    if notes == trancomments or trancomments.find(notes) >= 0:
-      notes = ""
-    elif notes.find(trancomments) >= 0:
-      trancomments = notes
-      notes = ""
-    trancomments = trancomments + notes
+    trancomments = transunit.getnotes("translator")
     if trancomments:
       thepo.othercomments.extend(["# %s\n" % comment for comment in trancomments.split("\n")])
     
-    #Automatic comments
-    autocomments = transunit.getautomaticcomments()
+    #Automatic and Developer comments
+    autocomments = transunit.getnotes("developer")
     if autocomments:
       thepo.automaticcomments.extend(["#. %s\n" % comment for comment in autocomments.split("\n")])
 
     #See 5.6.1 of the spec. We should not check fuzzyness, but approved attribute
-    if not transunit.isapproved():
+    if transunit.isfuzzy():
       thepo.markfuzzy(True)
     
     return thepo
 
   def convertfile(self, inputfile):
     """converts a .xliff file to .po format"""
-    PoXliffFile = xliff.PoXliffFile(inputfile)
+    # XXX: The inputfile is converted to string because Pootle supplies
+    # XXX: a PootleFile object as input which cannot be sent to PoXliffFile.
+    # XXX: The better way would be to have a consistent conversion API.
+    if not isinstance(inputfile, (file, wStringIO.StringIO)):
+        inputfile = str(inputfile)
+    XliffFile = xliff.xlifffile.parsestring(inputfile)
     thepofile = po.pofile()
     headerpo = thepofile.makeheader(charset="UTF-8", encoding="8bit")
-    # for filename, transunits in PoXliffFile.units:
     # TODO: support multiple files
-    if True:
-      for transunit in PoXliffFile.units:
+    for transunit in XliffFile.units:
         thepo = self.converttransunit(transunit)
         thepofile.units.append(thepo)
     return thepofile
@@ -88,7 +85,7 @@ def convertxliff(inputfile, outputfile, templates):
 
 def main(argv=None):
   from translate.convert import convert
-  formats = {"xliff":("po",convertxliff)}
+  formats = {"xlf":("po",convertxliff)}
   parser = convert.ConvertOptionParser(formats, usepots=True, description=__doc__)
   parser.run(argv)
 
