@@ -24,7 +24,9 @@
 from translate.filters import helpers
 from translate.filters import decoration
 from translate.filters import prefilters
-import sre
+from translate.misc.multistring import multistring
+from translate.lang import factory
+import re
 try:
   from enchant import checker
   checkers = {}
@@ -44,10 +46,19 @@ except ImportError:
   except ImportError:
     def dospellcheck(text, lang):
       return []
+
+def forceunicode(string):
+  """Helper method to ensure that the parameter becomes unicode if not yet"""
+  if string is None:
+    return None
+  if isinstance(string, str):
+    encoding = getattr(string, "encoding", "utf-8")
+    string = string.decode(encoding)
+  return string
     
 def tagname(string):
   """Returns the name of the XML/HTML tag in string"""
-  return sre.match("<[\s]*([\w\/]*)", string).groups(1)[0]
+  return re.match("<[\s]*([\w\/]*)", string).groups(1)[0]
 
 def intuplelist(pair, list):
   """Tests to see if pair == (a,b,c) is in list, but handles None entries in 
@@ -73,7 +84,7 @@ def tagproperties(strings, ignore):
     properties += [(tag, None, None)]
     #Now we isolate the attribute pairs. We allow escaped quotes
     #TODO: remove escaped strings once usage is audited
-    pairs = sre.findall(" (\w*)=((\\\\?\".*?\\\\?\")|(\\\\?'.*?\\\\?'))", string)
+    pairs = re.findall(" (\w*)=((\\\\?\".*?\\\\?\")|(\\\\?'.*?\\\\?'))", string)
     for property, value, a, b in pairs:
       #Strip the quotes:
       value = value[1:-1]
@@ -108,6 +119,8 @@ class SeriousFilterFailure(FilterFailure):
 
 def passes(filterfunction, str1, str2):
   """returns whether the given strings pass on the given test, handling FilterFailures"""
+  str1 = forceunicode(str1)
+  str2 = forceunicode(str2)
   try:
     filterresult = filterfunction(str1, str2)
   except FilterFailure, e:
@@ -115,18 +128,30 @@ def passes(filterfunction, str1, str2):
   return filterresult
 
 def fails(filterfunction, str1, str2):
-  """returns whether the given strings fail on the given test, handling FilterFailures"""
+  """returns whether the given strings fail on the given test, handling only FilterFailures"""
+  str1 = forceunicode(str1)
+  str2 = forceunicode(str2)
   try:
     filterresult = filterfunction(str1, str2)
+  except SeriousFilterFailure, e:
+    filterresult = True
   except FilterFailure, e:
     filterresult = False
   return not filterresult
 
-punctuation_chars = u'.,;:!?-@#$%^*_()[]{}/\\\'"<>\u2018\u2019\u201a\u201b\u201c\u201d\u201e\u201f\u2032\u2033\u2034\u2035\u2036\u2037\u2039\u203a\xab\xbb\xb1\xb3\xb9\xb2\xb0\xbf\xa9\xae\xd7\xa3\xa5\u2026'
-endpunctuation_chars = u'.:!?\u2026'
+def fails_serious(filterfunction, str1, str2):
+  """returns whether the given strings fail on the given test, handling only SeriousFilterFailures"""
+  str1 = forceunicode(str1)
+  str2 = forceunicode(str2)
+  try:
+    filterresult = filterfunction(str1, str2)
+  except SeriousFilterFailure, e:
+    filterresult = False
+  return not filterresult
+
 # printf syntax based on http://en.wikipedia.org/wiki/Printf which doens't cover everything we leave \w instead of specifying the exact letters as
 # this should capture printf types defined in other platforms.
-printf_pat = sre.compile('%((?:(?P<ord>\d+)\$)*(?P<fullvar>[+#-]*(?:\d+)*(?:\.\d+)*(hh\|h\|l\|ll)*(?P<type>[\w%])+))')
+printf_pat = re.compile('%((?:(?P<ord>\d+)\$)*(?P<fullvar>[+#-]*(?:\d+)*(?:\.\d+)*(hh\|h\|l\|ll)*(?P<type>[\w%])+))')
 
 #(tag, attribute, value) specifies a certain attribute which can be changed/
 #ignored if it exists inside tag. In the case where there is a third element
@@ -140,7 +165,7 @@ common_canchangetags = [("img", "alt", None)]
 
 class CheckerConfig(object):
   """object representing the configuration of a checker"""
-  def __init__(self, targetlanguage=None, accelmarkers=None, varmatches=None, notranslatewords=None, musttranslatewords=None, validchars=None, punctuation=None, endpunctuation=None, ignoretags=None, canchangetags=None):
+  def __init__(self, targetlanguage=None, accelmarkers=None, varmatches=None, notranslatewords=None, musttranslatewords=None, validchars=None, punctuation=None, endpunctuation=None, ignoretags=None, canchangetags=None, criticaltests=None):
     # make sure that we initialise empty lists properly (default arguments get reused!)
     if accelmarkers is None:
       accelmarkers = []
@@ -151,24 +176,23 @@ class CheckerConfig(object):
     if notranslatewords is None:
       notranslatewords = []
     self.targetlanguage = targetlanguage
+    self.updatetargetlanguage(targetlanguage)
+    self.sourcelang = factory.getlanguage('en')
     self.accelmarkers = accelmarkers
     self.varmatches = varmatches
     # TODO: allow user configuration of untranslatable words
     self.notranslatewords = dict.fromkeys([key for key in notranslatewords])
     self.musttranslatewords = dict.fromkeys([key for key in musttranslatewords])
-    if isinstance(validchars, str):
-      validchars = validchars.decode("utf-8")
+    validchars = forceunicode(validchars)
     self.validcharsmap = {}
     self.updatevalidchars(validchars)
-    if isinstance(punctuation, str):
-      punctuation = punctuation.decode("utf-8")
-    elif punctuation is None:
-      punctuation = punctuation_chars
+    punctuation = forceunicode(punctuation)
+    if punctuation is None:
+      punctuation = self.lang.punctuation
     self.punctuation = punctuation
-    if isinstance(endpunctuation, str):
-      endpunctuation = endpunctuation.decode("utf-8")
-    elif endpunctuation is None:
-      endpunctuation = endpunctuation_chars
+    endpunctuation = forceunicode(endpunctuation)
+    if endpunctuation is None:
+      endpunctuation = self.lang.sentenceend
     self.endpunctuation = endpunctuation
     if ignoretags is None:
       self.ignoretags = common_ignoretags
@@ -178,11 +202,14 @@ class CheckerConfig(object):
       self.canchangetags = common_canchangetags
     else:
       self.canchangetags = canchangetags
-    
-    
+    if criticaltests is None:
+      criticaltests = []
+    self.criticaltests = criticaltests
+
   def update(self, otherconfig):
     """combines the info in otherconfig into this config object"""
     self.targetlanguage = otherconfig.targetlanguage or self.targetlanguage
+    self.lang = otherconfig.lang
     self.accelmarkers.extend(otherconfig.accelmarkers)
     self.varmatches.extend(otherconfig.varmatches)
     self.notranslatewords.update(otherconfig.notranslatewords)
@@ -193,6 +220,8 @@ class CheckerConfig(object):
     #TODO: consider also updating in the following cases:
     self.ignoretags = otherconfig.ignoretags
     self.canchangetags = otherconfig.canchangetags
+    self.criticaltests.extend(otherconfig.criticaltests)
+
   def updatevalidchars(self, validchars):
     """updates the map that eliminates valid characters"""
     if validchars is None:
@@ -200,6 +229,10 @@ class CheckerConfig(object):
     validcharsmap = dict([(ord(validchar), None) for validchar in validchars])
     self.validcharsmap.update(validcharsmap)
 
+  def updatetargetlanguage(self, langcode):
+      """Updates the target language in the config to the given target language"""
+      self.lang = factory.getlanguage(langcode)
+
 class TranslationChecker(object):
   """Base Checker class which does the checking based on functions available in derived classes"""
   preconditions = {}
@@ -261,10 +294,16 @@ class TranslationChecker(object):
     """replaces words with punctuation with their unpunctuated equivalents..."""
     return prefilters.filterwordswithpunctuation(str1)
 
+  def filterxml(self, str1):
+    """filter out XML from the string so only text remains"""
+    return re.sub("<[^>]+>", "", str1)
+
   def run_filters(self, str1, str2):
     """run all the tests in this suite, return failures as testname, message_or_exception"""
+    str1 = forceunicode(str1)
+    str2 = forceunicode(str2)
     failures = []
-    ignores = []
+    ignores = self.config.lang.ignoretests[:]
     functionnames = self.defaultfilters.keys()
     priorityfunctionnames = self.preconditions.keys()
     otherfunctionnames = filter(lambda functionname: functionname not in self.preconditions, functionnames)
@@ -280,7 +319,7 @@ class TranslationChecker(object):
         filterresult = filterfunction(str1, str2)
       except FilterFailure, e:
         filterresult = False
-        filtermessage = e
+        filtermessage = str(e).decode('utf-8')
       except Exception, e:
         if self.errorhandler is None:
           raise
@@ -298,12 +337,15 @@ class TranslationChecker(object):
 
 class TeeChecker:
   """A Checker that controls multiple checkers..."""
-  def __init__(self, checkerconfig=None, excludefilters=None, limitfilters=None, checkerclasses=None, errorhandler=None):
+  def __init__(self, checkerconfig=None, excludefilters=None, limitfilters=None, checkerclasses=None, errorhandler=None, languagecode=None):
     """construct a TeeChecker from the given checkers"""
     self.limitfilters = limitfilters
     if checkerclasses is None:
       checkerclasses = [StandardChecker]
     self.checkers = [checkerclass(checkerconfig=checkerconfig, excludefilters=excludefilters, limitfilters=limitfilters, errorhandler=errorhandler) for checkerclass in checkerclasses]
+    if languagecode:
+        for checker in self.checkers:
+            checker.config.updatetargetlanguage(languagecode)
     self.combinedfilters = self.getfilters(excludefilters, limitfilters)
 
   def getfilters(self, excludefilters=None, limitfilters=None):
@@ -339,57 +381,77 @@ class StandardChecker(TranslationChecker):
 
   def unchanged(self, str1, str2):
     """checks whether a translation is basically identical to the original string"""
-    str1 = self.filteraccelerators(prefilters.removekdecomments(str1))
+    str1 = self.filteraccelerators(str1)
     str2 = self.filteraccelerators(str2)
+    if len(str1.strip()) == 0:
+      return True
+    if str1.isupper() and str1 == str2:
+      return True
     if self.config.notranslatewords:
       words1 = str1.split()
       if len(words1) == 1 and [word for word in words1 if word in self.config.notranslatewords]:
         return True
     str1 = self.removevariables(str1)
     str2 = self.removevariables(str2)
-    if not (str1.isdigit() or len(str1) < 2) and (str1.strip().lower() == str2.strip().lower()):
+    if not (str1.strip().isdigit() or len(str1) < 2 or decoration.ispurepunctuation(str1.strip())) and (str1.strip().lower() == str2.strip().lower()):
       raise FilterFailure("please translate")
     return True
 
   def blank(self, str1, str2):
-    """checks whether a translation is totally blank"""
-    len1 = len(prefilters.removekdecomments(str1).strip())
+    """checks whether a translation only contains spaces"""
+    len1 = len(str1.strip())
     len2 = len(str2.strip())
     return not (len1 > 0 and len(str2) != 0 and len2 == 0)
 
   def short(self, str1, str2):
     """checks whether a translation is much shorter than the original string"""
-    len1 = len(prefilters.removekdecomments(str1).strip())
+    len1 = len(str1.strip())
     len2 = len(str2.strip())
     return not ((len1 > 0) and (0 < len2 < (len1 * 0.1)) or ((len1 > 1) and (len2 == 1)))
 
   def long(self, str1, str2):
     """checks whether a translation is much longer than the original string"""
-    len1 = len(prefilters.removekdecomments(str1).strip())
+    len1 = len(str1.strip())
     len2 = len(str2.strip())
     return not ((len1 > 0) and (0 < len1 < (len2 * 0.1)) or ((len1 == 1) and (len2 > 1))) 
 
   def escapes(self, str1, str2):
     """checks whether escaping is consistent between the two strings"""
-    str1 = prefilters.removekdecomments(str1)
     if not helpers.countsmatch(str1, str2, ("\\", "\\\\")):
-      escapes1 = ", ".join(["'%s'" % word for word in str1.split() if "\\" in word])
-      escapes2 = ", ".join(["'%s'" % word for word in str2.split() if "\\" in word])
-      raise SeriousFilterFailure("escapes in original (%s) don't match escapes in translation (%s)" % (escapes1, escapes2))
+      escapes1 = u", ".join([u"'%s'" % word for word in str1.split() if "\\" in word])
+      escapes2 = u", ".join([u"'%s'" % word for word in str2.split() if "\\" in word])
+      raise SeriousFilterFailure(u"escapes in original (%s) don't match escapes in translation (%s)" % (escapes1, escapes2))
+    else:
+      return True
+
+  def newlines(self, str1, str2):
+    """checks whether newlines are consistent between the two strings"""
+    if not helpers.countsmatch(str1, str2, ("\n", "\r")):
+      raise FilterFailure("line endings in original don't match line endings in translation")
+    else:
+      return True
+
+  def tabs(self, str1, str2):
+    """checks whether tabs are consistent between the two strings"""
+    if not helpers.countmatch(str1, str2, "\t"):
+      raise SeriousFilterFailure("tabs in original don't match tabs in translation")
     else:
       return True
 
   def singlequoting(self, str1, str2):
     """checks whether singlequoting is consistent between the two strings"""
-    str1 = self.filterwordswithpunctuation(self.filteraccelerators(self.filtervariables(prefilters.removekdecomments(str1))))
+    str1 = self.filterwordswithpunctuation(self.filteraccelerators(self.filtervariables(str1)))
     str2 = self.filterwordswithpunctuation(self.filteraccelerators(self.filtervariables(str2)))
     return helpers.countsmatch(str1, str2, ("'", "''", "\\'"))
 
   def doublequoting(self, str1, str2):
     """checks whether doublequoting is consistent between the two strings"""
     str1 = self.filteraccelerators(self.filtervariables(str1))
+    str1 = self.filterxml(str1)
+    str1 = self.config.lang.punctranslate(str1)
     str2 = self.filteraccelerators(self.filtervariables(str2))
-    return helpers.countsmatch(str1, str2, ('"', '""', '\\"'))
+    str2 = self.filterxml(str2)
+    return helpers.countsmatch(str1, str2, ('"', '""', '\\"', u"«", u"»"))
 
   def doublespacing(self, str1, str2):
     """checks for bad double-spaces by comparing to original"""
@@ -400,6 +462,7 @@ class StandardChecker(TranslationChecker):
   def puncspacing(self, str1, str2):
     """checks for bad spacing after punctuation"""
     str1 = self.filteraccelerators(self.filtervariables(str1))
+    str1 = self.config.lang.punctranslate(str1)
     str2 = self.filteraccelerators(self.filtervariables(str2))
     for puncchar in self.config.punctuation:
       plaincount1 = str1.count(puncchar)
@@ -463,7 +526,10 @@ class StandardChecker(TranslationChecker):
       else:
         messages.append("accelerator %s occurs %d time(s) in original and %d time(s) in translation" % (accelmarker, count1, count2))
     if messages:
-      raise FilterFailure(messages)
+      if "accelerators" in self.config.criticaltests:
+        raise SeriousFilterFailure(messages)
+      else:
+        raise FilterFailure(messages)
     return True
 
 #  def acceleratedvariables(self, str1, str2):
@@ -482,7 +548,6 @@ class StandardChecker(TranslationChecker):
 
   def variables(self, str1, str2):
     """checks whether variables of various forms are consistent between the two strings"""
-    str1 = prefilters.removekdecomments(str1)
     messages = []
     mismatch1, mismatch2 = [], []
     varnames1, varnames2 = [], []
@@ -522,7 +587,7 @@ class StandardChecker(TranslationChecker):
 
   def functions(self, str1, str2):
     """checks to see that function names are not translated"""
-    return helpers.funcmatch(str1, str2, decoration.getfunctions)
+    return helpers.funcmatch(str1, str2, decoration.getfunctions, self.config.punctuation)
 
   def emails(self, str1, str2):
     """checks to see that emails are not translated"""
@@ -551,12 +616,14 @@ class StandardChecker(TranslationChecker):
   def startpunc(self, str1, str2):
     """checks whether punctuation at the beginning of the strings match"""
     str1 = self.filteraccelerators(self.filtervariables(self.filterwordswithpunctuation(str1)))
+    str1 = self.config.lang.punctranslate(str1)
     str2 = self.filteraccelerators(self.filtervariables(self.filterwordswithpunctuation(str2)))
     return helpers.funcmatch(str1, str2, decoration.puncstart, self.config.punctuation)
 
   def endpunc(self, str1, str2):
     """checks whether punctuation at the end of the strings match"""
     str1 = self.filteraccelerators(self.filtervariables(self.filterwordswithpunctuation(str1)))
+    str1 = self.config.lang.punctranslate(str1)
     str2 = self.filteraccelerators(self.filtervariables(self.filterwordswithpunctuation(str2)))
     return helpers.funcmatch(str1, str2, decoration.puncend, self.config.endpunctuation)
 
@@ -565,11 +632,12 @@ class StandardChecker(TranslationChecker):
     # this test is a subset of startandend
     if (decoration.ispurepunctuation(str1)):
       return str1 == str2
-    return 1
+    else:
+      return not decoration.ispurepunctuation(str2)
 
   def brackets(self, str1, str2):
     """checks that the number of brackets in both strings match"""
-    str1 = self.filtervariables(prefilters.removekdecomments(str1))
+    str1 = self.filtervariables(str1)
     str2 = self.filtervariables(str2)
     messages = []
     missing = []
@@ -591,15 +659,18 @@ class StandardChecker(TranslationChecker):
 
   def sentencecount(self, str1, str2):
     """checks that the number of sentences in both strings match"""
-    return helpers.countsmatch(prefilters.removekdecomments(str1), str2, ".")
+    sentences1 = len(self.config.sourcelang.sentences(str1))
+    sentences2 = len(self.config.lang.sentences(str2))
+    if not sentences1 == sentences2:
+        raise FilterFailure("The number of sentences differ: %d versus %d" % (sentences1, sentences2))
+    return True
 
   def startcaps(self, str1, str2):
     """checks that the message starts with the correct capitalisation"""
-    punc = self.config.punctuation
-    str1 = self.filteraccelerators(self.filtervariables(prefilters.removekdecomments(str1))).lstrip().lstrip(punc)
-    str2 = self.filteraccelerators(self.filtervariables(str2)).lstrip().lstrip(punc)
+    str1 = self.filteraccelerators(str1)
+    str2 = self.filteraccelerators(str2)
     if len(str1) > 1 and len(str2) > 1:
-      return str1[0].isupper() == str2[0].isupper()
+      return self.config.sourcelang.capsstart(str1) == self.config.lang.capsstart(str2)
     if len(str1) == 0 and len(str2) == 0:
       return True
     if len(str1) == 0 or len(str2) == 0:
@@ -608,11 +679,13 @@ class StandardChecker(TranslationChecker):
 
   def simplecaps(self, str1, str2):
     """checks the capitalisation of two strings isn't wildly different"""
-    str1 = self.removevariables(prefilters.removekdecomments(str1))
+    str1 = self.removevariables(str1)
     str2 = self.removevariables(str2)
-    str1 = sre.sub("[^.]( I )", " i ", str1)
-    capitals1, capitals2 = helpers.filtercount(str1, type(str1).isupper), helpers.filtercount(str2, type(str2).isupper)
-    alpha1, alpha2 = helpers.filtercount(str1, type(str1).isalpha), helpers.filtercount(str2, type(str2).isalpha)
+    str1 = re.sub(u"[^%s]( I )" % self.config.sourcelang.sentenceend, " i ", str1)
+    capitals1 = helpers.filtercount(str1, type(str1).isupper)
+    capitals2 = helpers.filtercount(str2, type(str2).isupper)
+    alpha1 = helpers.filtercount(str1, type(str1).isalpha)
+    alpha2 = helpers.filtercount(str2, type(str2).isalpha)
     # Capture the all caps case
     if capitals1 == alpha1:
       return capitals2 == alpha2
@@ -635,8 +708,9 @@ class StandardChecker(TranslationChecker):
     for startmatch, endmatch in self.config.varmatches:
       allowed += decoration.getvariables(startmatch, endmatch)(str1)
     allowed += self.config.musttranslatewords.keys()
-    for word in self.filteraccelerators(self.filtervariables(str1)).split():
-      word = word.strip(self.config.punctuation)
+    str1 = self.filteraccelerators(self.filtervariables(str1))
+    iter = self.config.lang.word_iter(str1)
+    for word in iter:
       if word.isupper() and len(word) > 1 and word not in allowed:
         if self.filteraccelerators(self.filtervariables(str2)).find(word) == -1:
           acronyms.append(word)
@@ -651,7 +725,7 @@ class StandardChecker(TranslationChecker):
     words = self.filteraccelerators(self.removevariables(without_newlines)).replace(".", "").lower().split()
     for word in words:
       if word == lastword:
-        return False
+        raise FilterFailure("The word '%s' is repeated" % word)
       lastword = word
     return True
 
@@ -701,10 +775,6 @@ class StandardChecker(TranslationChecker):
     """checks that only characters specified as valid appear in the translation"""
     if not self.config.validcharsmap:
       return True
-    if isinstance(str1, str):
-      str1 = str1.decode('utf-8')
-    if isinstance(str2, str):
-      str2 = str2.decode('utf-8')
     invalid1 = str1.translate(self.config.validcharsmap)
     invalid2 = str2.translate(self.config.validcharsmap)
     invalidchars = ["'%s' (\\u%04x)" % (invalidchar.encode('utf-8'), ord(invalidchar)) for invalidchar in invalid2 if invalidchar not in invalid1]
@@ -722,11 +792,12 @@ class StandardChecker(TranslationChecker):
 
   def xmltags(self, str1, str2):
     """checks that XML/HTML tags have not been translated"""
-    str1 = prefilters.removekdecomments(str1)
-    tags = sre.findall("<[^>]+>", str1)
-    if len(tags) > 0:
-      tags2 = sre.findall("<[^>]+>", str2)
-      properties1 = tagproperties(tags, self.config.ignoretags)
+    tags1 = re.findall("<[^>]+>", str1)
+    if len(tags1) > 0:
+      if (len(tags1[0]) == len(str1)) and not "=" in tags1[0]:
+        return True
+      tags2 = re.findall("<[^>]+>", str2)
+      properties1 = tagproperties(tags1, self.config.ignoretags)
       properties2 = tagproperties(tags2, self.config.ignoretags)
       filtered1 = []
       filtered2 = []
@@ -738,6 +809,12 @@ class StandardChecker(TranslationChecker):
       #TODO: consider the consequences of different ordering of attributes/tags
       if filtered1 != filtered2:
         return False
+    else:
+      # No tags in str1, let's just check that none were added in str2. This 
+      # might be useful for fuzzy strings wrongly unfuzzied, for example.
+      tags2 = re.findall("<[^>]+>", str2)
+      if len(tags2) > 0:
+        return False
     return True
 
   def kdecomments(self, str1, str2):
@@ -750,7 +827,30 @@ class StandardChecker(TranslationChecker):
 
   def simpleplurals(self, str1, str2):
     """checks for English style plural(s) for you to review"""
-    return str1.find("(s)") == -1
+    def numberofpatterns(string, patterns):
+      number = 0
+      for pattern in patterns:
+        number += len(re.findall(pattern, string))
+      return number
+
+    sourcepatterns = ["\(s\)"]
+    targetpatterns = ["\(s\)"]
+    sourcecount = numberofpatterns(str1, sourcepatterns)
+    targetcount = numberofpatterns(str2, targetpatterns)
+    if self.config.lang.nplurals == 1:
+        return not targetcount
+    return sourcecount == targetcount
+
+  def nplurals(self, str1, str2):
+    """Checks for the correct number of noun forms for plural translations."""
+    if isinstance(str1, multistring) and isinstance(str2, multistring):
+      # if we don't have a valid nplurals value, don't run the test
+      if len(str1.strings) > 1 and self.config.lang.nplurals != 0:
+        return len(str2.strings) == self.config.lang.nplurals
+      else:
+        return True
+    else:
+      return True
 
   def spellcheck(self, str1, str2):
     """checks words that don't pass a spell check"""
@@ -785,8 +885,21 @@ class StandardChecker(TranslationChecker):
                                     "filepaths", "purepunc", "doublespacing",
                                     "sentencecount", "numbers", "isfuzzy",
                                     "isreview", "notranslatewords", "musttranslatewords",
-                                    "emails", "simpleplurals", "urls", "printf"),
-                   "unchanged": ("doublewords",), 
+                                    "emails", "simpleplurals", "urls", "printf",
+                                    "tabs", "newlines", "blank", "nplurals"),
+                    "blank":        ("simplecaps", "variables", "startcaps",
+                                    "accelerators", "brackets", "endpunc",
+                                    "acronyms", "xmltags", "startpunc",
+                                    "endwhitespace", "startwhitespace",
+                                    "escapes", "doublequoting", "singlequoting", 
+                                    "filepaths", "purepunc", "doublespacing",
+                                    "sentencecount", "numbers", "isfuzzy",
+                                    "isreview", "notranslatewords", "musttranslatewords",
+                                    "emails", "simpleplurals", "urls", "printf",
+                                    "tabs", "newlines"),
+                   "startcaps":     ("simplecaps",),
+                   "endwhitespace": ("endpunc",),
+                   "unchanged":     ("doublewords",), 
                    "compendiumconflicts": ("accelerators", "brackets", "escapes", 
                                     "numbers", "startpunc", "long", "variables", 
                                     "startcaps", "sentencecount", "simplecaps",
@@ -815,7 +928,8 @@ class OpenOfficeChecker(StandardChecker):
 
 mozillaconfig = CheckerConfig(
   accelmarkers = ["&"],
-  varmatches = [("&", ";"), ("%", "%"), ("%", 1), ("$", "$"), ("$", None), ("#", 1), ("${", "}"), ("$(^", ")")]
+  varmatches = [("&", ";"), ("%", "%"), ("%", 1), ("$", "$"), ("$", None), ("#", 1), ("${", "}"), ("$(^", ")")],
+  criticaltests = ["accelerators"]
   )
 
 class MozillaChecker(StandardChecker):
@@ -857,11 +971,22 @@ class KdeChecker(StandardChecker):
     checkerconfig.update(kdeconfig)
     StandardChecker.__init__(self, **kwargs)
 
+cclicenseconfig = CheckerConfig(varmatches = [("@", "@")])
+class CCLicenseChecker(StandardChecker):
+  def __init__(self, **kwargs):
+    checkerconfig = kwargs.get("checkerconfig", None)
+    if checkerconfig is None:
+      checkerconfig = CheckerConfig()
+      kwargs["checkerconfig"] = checkerconfig
+    checkerconfig.update(cclicenseconfig)
+    StandardChecker.__init__(self, **kwargs)
+
 projectcheckers = {
   "openoffice": OpenOfficeChecker,
   "mozilla": MozillaChecker,
   "kde": KdeChecker,
-  "gnome": GnomeChecker
+  "gnome": GnomeChecker,
+  "creativecommons": CCLicenseChecker
   }
 
 def runtests(str1, str2, ignorelist=()):
