@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # 
-# Copyright 2004-2006 Zuza Software Foundation
+# Copyright 2004-2007 Zuza Software Foundation
 # 
 # This file is part of translate.
 #
@@ -21,7 +21,8 @@
 
 """functions to get decorative/informative text out of strings..."""
 
-import sre
+import re
+import unicodedata
 
 def spacestart(str1):
   """returns all the whitespace from the start of the string"""
@@ -61,14 +62,22 @@ def ispurepunctuation(str1):
   """checks whether the string is entirely punctuation"""
   for c in str1:
     if c.isalpha(): return 0
-  return 1
+  return len(str1)
 
 def isvalidaccelerator(accelerator, ignorelist=[]):
   """returns whether the given accelerator string is a valid one..."""
   if len(accelerator) == 0 or accelerator in ignorelist:
     return 0
   accelerator = accelerator.replace("_","")
-  return accelerator.isalnum()
+  if not accelerator.isalnum():
+      return False
+  
+  # We don't want to have accelerators on characters with diacritics, so let's 
+  # se if the character can decompose.
+  decomposition = unicodedata.decomposition(accelerator)
+  # Next we strip out any extra information like <this>
+  decomposition = re.sub("<[^>]+>", "", decomposition).strip()
+  return decomposition.count(" ") == 0
 
 def findaccelerators(str1, accelmarker, ignorelist=[]):
   """returns all the accelerators and locations in str1 marked with a given marker"""
@@ -102,14 +111,15 @@ def findmarkedvariables(str1, startmarker, endmarker, ignorelist=[]):
       startmatch = currentpos
       currentpos += len(startmarker)
       if endmarker is None:
-        # handle case without an end marker - use any non-alphanumeric character
+        # handle case without an end marker - use any non-alphanumeric character as the end marker, var must be len > 1
         endmatch = currentpos
         for n in range(currentpos, len(str1)):
           if not str1[n].isalnum():
             endmatch = n
             break
         if currentpos == endmatch: endmatch = len(str1)
-        variable = str1[currentpos:endmatch]
+        if currentpos < endmatch:
+          variable = str1[currentpos:endmatch]
         currentpos = endmatch
       elif type(endmarker) == int:
         # setting endmarker to an int means it is a fixed-length variable string (usually endmarker==1)
@@ -194,19 +204,21 @@ def getnumbers(str1):
     numbers = [number.encode('utf8') for number in numbers]
   return numbers
 
-def getfunctions(str1):
-  """returns the functions() that are in a string"""
-  return [word for word in str1.split() if word.endswith("()")]
+def getfunctions(str1, punctuation):
+  """returns the functions() that are in a string, while ignoring the trailing 
+  punctuation in the given parameter"""
+  punctuation = punctuation.replace("(", "").replace(")", "")
+  return [word.rstrip(punctuation) for word in str1.split() if word.rstrip(punctuation).endswith("()")]
 
 def getemails(str1):
   """returns the email addresses that are in a string"""
-  return sre.findall('[\w\.\-]+@[\w\.\-]+', str1)
+  return re.findall('[\w\.\-]+@[\w\.\-]+', str1)
 
 def geturls(str1):
   """returns the URIs in a string"""
   URLPAT = 'https?:[\w/\.:;+\-~\%#\$?=&,()]+|www\.[\w/\.:;+\-~\%#\$?=&,()]+|' +\
            'ftp:[\w/\.:;+\-~\%#?=&,]+'
-  return sre.findall(URLPAT, str1)
+  return re.findall(URLPAT, str1)
 
 
 def countaccelerators(accelmarker, ignorelist=[]):
