@@ -20,7 +20,7 @@
 
 """simple script to run all check filters on gettext .po localization file(s)"""
 
-from translate.storage import po
+from translate.storage import factory
 from translate.filters import checks
 from translate.filters import autocorrect
 from translate.misc import optrecurse
@@ -78,12 +78,12 @@ class StandardPOChecker(POChecker):
 
   def isreview(self, thepo):
     """check if the po element has been marked review"""
-    return not (thepo.isreview() or thepo.hasmarkedcomment("pofilter"))
+    return not thepo.isreview()
 
 class pocheckfilter:
   def __init__(self, options, checkerclasses=None, checkerconfig=None):
     # excludefilters={}, limitfilters=None, includeheader=False, includefuzzy=True, includereview=True, autocorrect=False):
-    """builds a pocheckfilter using the given checker (a list is allowed too)"""
+    """builds a checkfilter using the given checker (a list is allowed too)"""
     if checkerclasses is None:
       checkerclasses = [checks.StandardChecker, StandardPOChecker]
     self.checker = POTeeChecker(checkerconfig=checkerconfig, excludefilters=options.excludefilters, limitfilters=options.limitfilters, checkerclasses=checkerclasses)
@@ -120,40 +120,25 @@ class pocheckfilter:
           return []
     return failures
 
-  def filterfile(self, thepofile):
-    """runs filters on a file"""
-    thenewpofile = po.pofile()
-    for thepo in thepofile.units:
-      filterresult = self.filterelement(thepo)
+  def filterfile(self, transfile):
+    """Runs filters on a translation store object.
+    Parameters:
+      - transfile. A translation store object.
+    Return value:
+      - A new translation store object with the results of the filter included."""
+    newtransfile = type(transfile)()
+    for unit in transfile.units:
+      filterresult = self.filterelement(unit)
       if filterresult:
         if filterresult != autocorrect:
           for filtername, filtermessage in filterresult:
-            if filtername == "isreview":
-              if thepo.hastypecomment("review"):
-                reviewcomments = [comment for comment in thepo.typecomments if "review" in comment.replace("#,", "", 1).strip()]
-                for comment in reviewcomments:
-                  reviewmessage = comment.replace("#,", "", 1).strip()
-                  if reviewmessage == "review":
-                    reviewmessage = "(review)"
-                    thepo.typecomments = [typecomment for typecomment in thepo.typecomments if typecomment != comment]
-                  elif reviewmessage.startswith("review - "):
-                    reviewmessage = reviewmessage.replace("review - ", "(review) ", 1)
-                    thepo.typecomments = [typecomment for typecomment in thepo.typecomments if typecomment != comment]
-                  else:
-                    reviewmessage = "(review) " + reviewmessage
-                  thepo.othercomments.append("# %s\n" % reviewmessage)
-                thepo.settypecomment("review", False)
-            else:
-              newcomment = "# (pofilter) %s: %s\n" % (filtername, filtermessage)
-              if newcomment not in thepo.othercomments:
-                thepo.othercomments.append(newcomment)
-          for filtername, filtermessage in filterresult:
+            unit.adderror(filtername, filtermessage)
             if isinstance(filtermessage, checks.SeriousFilterFailure):
-              thepo.markfuzzy()
-        thenewpofile.units.append(thepo)
-    if self.options.includeheader and thenewpofile.units > 0:
-      thenewpofile.units.insert(0, thenewpofile.makeheader("UTF-8", "8bit"))
-    return thenewpofile
+              unit.markfuzzy()
+        newtransfile.addunit(unit)
+    if self.options.includeheader and newtransfile.units > 0:
+      newtransfile.units.insert(0, newtransfile.makeheader("UTF-8", "8bit"))
+    return newtransfile
 
 class FilterOptionParser(optrecurse.RecursiveOptionParser):
   """a specialized Option Parser for filter tools..."""
@@ -206,9 +191,9 @@ class FilterOptionParser(optrecurse.RecursiveOptionParser):
     else:
       self.recursiveprocess(options)
 
-def runpofilter(inputfile, outputfile, templatefile, checkfilter=None):
-  """reads in inputfile using po.pofile, filters using pocheckfilter, writes to stdout"""
-  fromfile = po.pofile(inputfile)
+def runfilter(inputfile, outputfile, templatefile, checkfilter=None):
+  """reads in inputfile, filters using checkfilter, writes to outputfile"""
+  fromfile = factory.getobject(inputfile)
   tofile = checkfilter.filterfile(fromfile)
   if tofile.isempty():
     return 0
@@ -216,7 +201,8 @@ def runpofilter(inputfile, outputfile, templatefile, checkfilter=None):
   return 1
 
 def cmdlineparser():
-  formats = {"po":("po", runpofilter), "pot":("pot", runpofilter), None:("po", runpofilter)}
+  formats = {"po":("po", runfilter), "pot":("pot", runfilter), "xliff":("xliff", runfilter), "xlf":("xlf", runfilter), "xlff":("xlff", runfilter), None:("po", runfilter)}
+
   parser = FilterOptionParser(formats)
   parser.add_option("", "--review", dest="includereview",
     action="store_true", default=True,
@@ -237,7 +223,7 @@ def cmdlineparser():
     action="store_true", default=False,
     help="output automatic corrections where possible rather than describing issues")
   parser.add_option("", "--language", dest="targetlanguage", default=None, 
-                    help="set target language code (e.g. af-ZA) [required for spell check]", metavar="LANG")
+                    help="set target language code (e.g. af-ZA) [required for spell check and recommended in general]", metavar="LANG")
   parser.add_option("", "--openoffice", dest="filterclass",
     action="store_const", default=None, const=checks.OpenOfficeChecker,
     help="use the standard checks for OpenOffice translations")
@@ -266,9 +252,12 @@ def cmdlineparser():
     default=None, type="string", metavar="FILE",
     help="read list of all valid characters from FILE (must be in UTF-8)")
   parser.passthrough.append('checkfilter')
+  parser.description=__doc__
   return parser
 
 def main():
   parser = cmdlineparser()
   parser.run()
 
+if __name__ == '__main__':
+    main()
