@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from translate.filters import checks
+from translate.misc.multistring import multistring
 
 def test_defaults():
     """tests default setup and that checks aren't altered by other constructions"""
@@ -50,14 +51,18 @@ def test_accelerators():
     assert checks.fails(gnomechecker.accelerators, "File", "_Fayile") 
     mozillachecker = checks.MozillaChecker()
     assert checks.passes(mozillachecker.accelerators, "&File", "&Fayile") 
-    assert checks.fails(mozillachecker.accelerators, "&File", "Fayile") 
-    assert checks.fails(mozillachecker.accelerators, "File", "&Fayile") 
+    assert checks.fails_serious(mozillachecker.accelerators, "&File", "Fayile") 
+    assert checks.fails_serious(mozillachecker.accelerators, "File", "&Fayile") 
     assert checks.passes(mozillachecker.accelerators, "Mail &amp; News", "Pos en Nuus") 
-    assert checks.fails(mozillachecker.accelerators, "Mail &amp; News", "Pos en &Nuus") 
+    assert checks.fails_serious(mozillachecker.accelerators, "Mail &amp; News", "Pos en &Nuus") 
+    assert checks.fails_serious(mozillachecker.accelerators, "&File", "Fayile") 
     ooochecker = checks.OpenOfficeChecker()
     assert checks.passes(ooochecker.accelerators, "~File", "~Fayile") 
     assert checks.fails(ooochecker.accelerators, "~File", "Fayile") 
     assert checks.fails(ooochecker.accelerators, "File", "~Fayile") 
+
+    # We don't want an accelerator for letters with a diacritic
+    assert checks.fails(ooochecker.accelerators, "F~ile", "L~êer")
     # Problems:
     # Accelerator before variable - see test_acceleratedvariables
 
@@ -77,6 +82,7 @@ def test_accronyms():
     stdchecker = checks.StandardChecker()
     assert checks.passes(stdchecker.acronyms, "An HTML file", "'n HTML leer")
     assert checks.fails(stdchecker.acronyms, "An HTML file", "'n LMTH leer")
+    assert checks.passes(stdchecker.acronyms, "It is HTML.", "Dit is HTML.")
     # We don't mind if you add an acronym to correct bad capitalisation in the original
     assert checks.passes(stdchecker.acronyms, "An html file", "'n HTML leer")
     # We shouldn't worry about acronyms that appear in a musttranslate file
@@ -116,6 +122,13 @@ def test_doublequoting():
     assert checks.passes(stdchecker.doublequoting, "\"Hot\" plate", "\"Ipuleti\" elishisa")
     assert checks.fails(stdchecker.doublequoting, "'Hot' plate", "\"Ipuleti\" elishisa")
     assert checks.passes(stdchecker.doublequoting, "\\\"Hot\\\" plate", "\\\"Ipuleti\\\" elishisa")
+
+    # We don't want the filter to complain about "untranslated" quotes in xml attributes
+    frchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="fr"))
+    assert checks.passes(frchecker.doublequoting, "Click <a href=\"page.html\">", "Clique <a href=\"page.html\">")
+    assert checks.fails(frchecker.doublequoting, "Do \"this\"", "Do \"this\"")
+    assert checks.passes(frchecker.doublequoting, "Do \"this\"", "Do « this »")
+    assert checks.fails(frchecker.doublequoting, "Do \"this\"", "Do « this » « this »")
     
 def test_doublespacing():
     """tests double spacing"""
@@ -152,6 +165,12 @@ def test_endpunc():
     # Making sure singlequotes don't confuse things
     assert checks.passes(stdchecker.endpunc, "Pseudo-elements can't be negated '%1$S'.", "Pseudo-elemente kan nie '%1$S' ontken word nie.")
 
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='km'))
+    assert checks.passes(stdchecker.endpunc, "In this new version, there are some minor conversion improvements on complex style in Openoffice.org Writer.", u"នៅ​ក្នុង​កំណែ​ថ្មីនេះ មាន​ការ​កែសម្រួល​មួយ​ចំនួន​តូច​ទាក់​ទង​នឹង​ការ​បំលែង​ពុម្ពអក្សរ​ខ្មែរ​ ក្នុង​កម្មវិធី​ការិយាល័យ​ ស្លឹករឹត ដែល​មាន​ប្រើ​ប្រាស់​រចនាប័ទ្មស្មុគស្មាញច្រើន\u00a0។")
+
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='zh'))
+    assert checks.passes(stdchecker.endpunc, "To activate your account, follow this link:\n", u"要啟用戶口，請瀏覽這個鏈結：\n")
+
 def test_endwhitespace():
     """tests endwhitespace"""
     stdchecker = checks.StandardChecker()
@@ -161,21 +180,50 @@ def test_endwhitespace():
 def test_escapes():
     """tests escapes"""
     stdchecker = checks.StandardChecker()
-    assert checks.passes(stdchecker.escapes, r"""_: KDE comment\n
-A sentence""", "I'm correct.")
-    assert checks.passes(stdchecker.escapes, r"A file\n", r"'n Leer\n")
-    assert checks.fails(stdchecker.escapes, r"A file\n", r"'n Leer")
-    assert checks.fails(stdchecker.escapes, r"blah.\nA file", r"bleah. 'n leer")
-    assert checks.fails(stdchecker.escapes, r"blah. A file", r"bleah.\n'n leer")
+    assert checks.passes(stdchecker.escapes, r"""A sentence""", "I'm correct.")
+    assert checks.passes(stdchecker.escapes, "A file\n", "'n Leer\n")
+    assert checks.fails_serious(stdchecker.escapes, r"blah. A file", r"bleah.\n'n leer")
     assert checks.passes(stdchecker.escapes, r"A tab\t", r"'n Tab\t")
-    assert checks.fails(stdchecker.escapes, r"A tab\t", r"'n Tab")
+    assert checks.fails_serious(stdchecker.escapes, r"A tab\t", r"'n Tab")
     assert checks.passes(stdchecker.escapes, r"An escape escape \\", r"Escape escape \\")
-    assert checks.fails(stdchecker.escapes, r"An escape escape \\", r"Escape escape")
+    assert checks.fails_serious(stdchecker.escapes, r"An escape escape \\", "Escape escape")
     assert checks.passes(stdchecker.escapes, r"A double quote \"", r"Double quote \"")
-    assert checks.fails(stdchecker.escapes, r"A double quote \"", r"Double quote")
+    assert checks.fails_serious(stdchecker.escapes, r"A double quote \"", "Double quote")
     # Escaped escapes
-    assert checks.passes(stdchecker.escapes, r"An escaped newline \\n", r"Escaped newline \\n")
-    assert checks.fails(stdchecker.escapes, r"An escaped newline \\n", r"Escaped newline \n")
+    assert checks.passes(stdchecker.escapes, "An escaped newline \\n", "Escaped newline \\n")
+    assert checks.fails_serious(stdchecker.escapes, "An escaped newline \\n", "Escaped newline \n")
+    # Real example
+    ooochecker = checks.OpenOfficeChecker()
+    assert checks.passes(ooochecker.escapes, ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32", ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32")
+
+def test_newlines():
+    """tests newlines"""
+    stdchecker = checks.StandardChecker()
+    assert checks.passes(stdchecker.newlines, "Nothing to see", "Niks te sien")
+    assert checks.passes(stdchecker.newlines, "Correct\n", "Korrek\n")
+    assert checks.passes(stdchecker.newlines, "Correct\r", "Korrek\r")
+    assert checks.passes(stdchecker.newlines, "Correct\r\n", "Korrek\r\n")
+    assert checks.fails(stdchecker.newlines, "A file\n", "'n Leer")
+    assert checks.fails(stdchecker.newlines, "A file", "'n Leer\n")
+    assert checks.fails(stdchecker.newlines, "A file\r", "'n Leer")
+    assert checks.fails(stdchecker.newlines, "A file", "'n Leer\r")
+    assert checks.fails(stdchecker.newlines, "A file\n", "'n Leer\r\n")
+    assert checks.fails(stdchecker.newlines, "A file\r\n", "'n Leer\n")
+    assert checks.fails(stdchecker.newlines, "blah.\nA file", "bleah. 'n leer")
+    # Real example
+    ooochecker = checks.OpenOfficeChecker()
+    assert checks.fails(ooochecker.newlines, "The arrowhead was modified without saving.\nWould you like to save the arrowhead now?", "Ṱhoho ya musevhe yo khwinifhadzwa hu si na u seiva.Ni khou ṱoda u seiva thoho ya musevhe zwino?")
+
+def test_tabs():
+    """tests tabs"""
+    stdchecker = checks.StandardChecker()
+    assert checks.passes(stdchecker.tabs, "Nothing to see", "Niks te sien")
+    assert checks.passes(stdchecker.tabs, "Correct\t", "Korrek\t")
+    assert checks.passes(stdchecker.tabs, "Correct\tAA", "Korrek\tAA")
+    assert checks.fails_serious(stdchecker.tabs, "A file\t", "'n Leer")
+    assert checks.fails_serious(stdchecker.tabs, "A file", "'n Leer\t")
+    ooochecker = checks.OpenOfficeChecker()
+    assert checks.passes(ooochecker.tabs, ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32", ",\t44\t;\t59\t:\t58\t{Tab}\t9\t{space}\t32")
 
 def test_filepaths():
     """tests filepaths"""
@@ -191,6 +239,7 @@ A string to translate""", "'n String om te vertaal")
     assert checks.fails(stdchecker.kdecomments, r"""_: I am a comment\n
 A string to translate""", r"""_: Ek is 'n commment\n
 'n String om te vertaal""")
+    assert checks.fails(stdchecker.kdecomments, """_: I am a comment\\n\n""", """_: I am a comment\\n\n""")
 
 def test_long():
     """tests long messages"""
@@ -247,7 +296,6 @@ def test_notranslatewords():
     assert checks.fails(stdchecker.notranslatewords, "Click Mozilla!", "Kliek Motzille!")
     assert checks.passes(stdchecker.notranslatewords, "Click Mozilla!", "Kliek Mozilla!")
     assert checks.fails(stdchecker.notranslatewords, "Searches (From Opera)", "adosako (kusukela ku- Ophera)")
-    assert checks.fails(stdchecker.notranslatewords, "Searches (From Opera)", "adosako (kusukela ku- Ophera)")
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=["Sun","NeXT"]))
     assert checks.fails(stdchecker.notranslatewords, "Sun/NeXT Audio", "Odio dza Ḓuvha/TeVHELAHO")
     assert checks.passes(stdchecker.notranslatewords, "Sun/NeXT Audio", "Odio dza Sun/NeXT")
@@ -306,18 +354,27 @@ def test_puncspacing():
     assert checks.fails(stdchecker.puncspacing, "One, two, three. ", "Kunye, kubili,kuthathu.")
     assert checks.passes(stdchecker.puncspacing, "One, two, three!?", "Kunye, kubili, kuthathu?")
 
+    # Some languages have padded puntuation marks
+    frchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="fr"))
+    assert checks.passes(frchecker.puncspacing, "Do \"this\"", "Do « this »")
+    assert checks.fails(frchecker.puncspacing, "Do \"this\"", "Do «this»")
+
 def test_purepunc():
     """tests messages containing only punctuation"""
     stdchecker = checks.StandardChecker()
     assert checks.passes(stdchecker.purepunc, ".", ".")
+    assert checks.passes(stdchecker.purepunc, "", "")
     assert checks.fails(stdchecker.purepunc, ".", " ")
+    assert checks.fails(stdchecker.purepunc, "Find", "'")
+    assert checks.fails(stdchecker.purepunc, "'", "Find")
 
 def test_sentencecount():
     """tests sentencecount messages"""
     stdchecker = checks.StandardChecker()
     assert checks.passes(stdchecker.sentencecount, "One. Two. Three.", "Een. Twee. Drie.")
-    assert checks.fails(stdchecker.sentencecount, "One two three", "Een twee drie.")
+    assert checks.passes(stdchecker.sentencecount, "One two three", "Een twee drie.")
     assert checks.fails(stdchecker.sentencecount, "One. Two. Three.", "Een Twee. Drie.")
+    assert checks.passes(stdchecker.sentencecount, "Sentence with i.e. in it.", "Sin met d.w.s. in dit.") # bug 178, description item 8
 
 def test_short():
     """tests short messages"""
@@ -336,9 +393,6 @@ def test_singlequoting():
     assert checks.passes(stdchecker.singlequoting, "File '%s'\n", "'%s' Faele\n")
     assert checks.fails(stdchecker.singlequoting, "'Hot' plate", "Ipuleti \"elishisa\"")
     assert checks.passes(stdchecker.singlequoting, "It's here.", "Dit is hier.")
-    # We shouldn't see single quotes in KDE comments
-    assert checks.passes(stdchecker.singlequoting, r"""_: 'Migrating' formats.\n
-Converting...""", "Kugucula...")
     # Don't get confused by punctuation that touches a single quote
     assert checks.passes(stdchecker.singlequoting, "File '%s'.", "'%s' Faele.")
     assert checks.passes(stdchecker.singlequoting, "Blah 'format' blah.", "Blah blah 'sebopego'.")
@@ -365,6 +419,7 @@ def test_simplecaps():
     # We should squash 'I' in the source text as it messes with capital detection
     assert checks.passes(stdchecker.simplecaps, "if you say I want", "as jy se ek wil")
     assert checks.passes(stdchecker.simplecaps, "sentence. I want more.", "sin. Ek wil meer he.")
+    assert checks.passes(stdchecker.simplecaps, "Where are we? I can't see where we are going.", "Waar is ons? Ek kan nie sien waar ons gaan nie.")
     ## We should remove variables before checking
     stdchecker = checks.StandardChecker(checks.CheckerConfig(varmatches=[("%", 1)]))
     assert checks.passes(stdchecker.simplecaps, "Could not load %s", "A swi koteki ku panga %S")
@@ -373,13 +428,12 @@ def test_simplecaps():
     assert checks.passes(stdchecker.simplecaps, "Determine how &brandShortName; connects to the Internet.", "Kuma &brandShortName; hlanganisa eka Internete.")
     ## If source is ALL CAPS then we should just check that target is also ALL CAPS
     assert checks.passes(stdchecker.simplecaps, "COUPDAYS", "COUPMALANGA")
-    # KDE commens should be removed
-    assert checks.passes(stdchecker.simplecaps, "_: KDE COMMENTS\\n\nA string", "Dimpled ring")
     # Just some that at times have failed but should always pass
     assert checks.passes(stdchecker.simplecaps, "Create a query  entering an SQL statement directly.", "Yakha sibuti singena SQL inkhomba yesitatimende.")
     ooochecker = checks.OpenOfficeChecker()
     assert checks.passes(ooochecker.simplecaps, "SOLK (%PRODUCTNAME Link)", "SOLK (%PRODUCTNAME Thumanyo)")
     assert checks.passes(ooochecker.simplecaps, "%STAROFFICE Image", "Tshifanyiso tsha %STAROFFICE")
+    assert checks.passes(stdchecker.simplecaps, "Flies, flies, everywhere! Ack!", u"Vlieë, oral vlieë! Jig!")
 
 def test_spellcheck():
     """tests spell checking"""
@@ -394,8 +448,6 @@ def test_startcaps():
     assert checks.passes(stdchecker.startcaps, "find", "vind")
     assert checks.fails(stdchecker.startcaps, "Find", "vind")
     assert checks.fails(stdchecker.startcaps, "find", "Vind")
-    assert checks.fails(stdchecker.startcaps, "Find", "'")
-    assert checks.fails(stdchecker.startcaps, "'", "Find")
     assert checks.passes(stdchecker.startcaps, "'", "'")
     assert checks.passes(stdchecker.startcaps, "\\.,/?!`'\"[]{}()@#$%^&*_-;:<>Find", "\\.,/?!`'\"[]{}()@#$%^&*_-;:<>Vind")
     # With leading whitespace
@@ -421,6 +473,11 @@ def test_startcaps():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&"))
     assert checks.passes(stdchecker.startcaps, "&Find", "Vi&nd")
 
+    # Language specific stuff
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='af'))
+    assert checks.passes(stdchecker.startcaps, "A cow", "'n Koei")
+    assert checks.fails(stdchecker.startcaps, "a cow", "'n Koei")
+
 def test_startpunc():
     """tests startpunc"""
     stdchecker = checks.StandardChecker()
@@ -441,15 +498,20 @@ def test_unchanged():
     assert checks.fails(stdchecker.unchanged, "&Unchanged", "Un&changed") 
     assert checks.passes(stdchecker.unchanged, "Unchanged", "Changed") 
     assert checks.passes(stdchecker.unchanged, "1234", "1234") 
+    assert checks.passes(stdchecker.unchanged, "2×2", "2×2") # bug 178, description item 14
     assert checks.passes(stdchecker.unchanged, "I", "I") 
-    assert checks.fails(stdchecker.unchanged, r"""_: KDE comment\n
-Unchanged""", r"Unchanged") 
-    # Variable only messages should be ignored
+    assert checks.passes(stdchecker.unchanged, "   ", "   ")  # bug 178, description item 5
+    assert checks.passes(stdchecker.unchanged, "???", "???")  # bug 178, description item 15
+    assert checks.passes(stdchecker.unchanged, "&ACRONYM", "&ACRONYM") # bug 178, description item 7
+    assert checks.passes(stdchecker.unchanged, "F1", "F1") # bug 178, description item 20
+    # Variable only and variable plus punctuation messages should be ignored
     mozillachecker = checks.MozillaChecker()
     assert checks.passes(mozillachecker.unchanged, "$ProgramName$", "$ProgramName$") 
+    assert checks.passes(mozillachecker.unchanged, "$file$ : $dir$", "$file$ : $dir$") # bug 178, description item 13
+    assert checks.fails(mozillachecker.unchanged, "$file$ in $dir$", "$file$ in $dir$") 
     # Don't translate words should be ignored
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=["Mozilla"]))
-    assert checks.passes(stdchecker.unchanged, "Mozilla", "Mozilla") 
+    assert checks.passes(stdchecker.unchanged, "Mozilla", "Mozilla") # bug 178, description item 10
 
 def test_untranslated():
     """tests untranslated entries"""
@@ -475,90 +537,99 @@ def test_variables_kde():
     # GNOME variables
     kdechecker = checks.KdeChecker()
     assert checks.passes(kdechecker.variables, "%d files of type %s saved.", "%d leers van %s tipe gestoor.")
-    assert checks.fails(kdechecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
+    assert checks.fails_serious(kdechecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
 
 def test_variables_gnome():
     """tests variables in GNOME translations"""
     # GNOME variables
     gnomechecker = checks.GnomeChecker()
     assert checks.passes(gnomechecker.variables, "%d files of type %s saved.", "%d leers van %s tipe gestoor.")
-    assert checks.fails(gnomechecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
+    assert checks.fails_serious(gnomechecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
     assert checks.passes(gnomechecker.variables, "Save $(file)", "Stoor $(file)")
-    assert checks.fails(gnomechecker.variables, "Save $(file)", "Stoor $(leer)")
+    assert checks.fails_serious(gnomechecker.variables, "Save $(file)", "Stoor $(leer)")
 
 def test_variables_mozilla():
     """tests variables in Mozilla translations"""
     # Mozilla variables
     mozillachecker = checks.MozillaChecker()
     assert checks.passes(mozillachecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandShortname; weergawe.")
-    assert checks.fails(mozillachecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandKortnaam; weergawe.")
+    assert checks.fails_serious(mozillachecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandKortnaam; weergawe.")
     assert checks.passes(mozillachecker.variables, "Save %file%", "Stoor %file%")
-    assert checks.fails(mozillachecker.variables, "Save %file%", "Stoor %leer%")
+    assert checks.fails_serious(mozillachecker.variables, "Save %file%", "Stoor %leer%")
     assert checks.passes(mozillachecker.variables, "Save $file$", "Stoor $file$")
-    assert checks.fails(mozillachecker.variables, "Save $file$", "Stoor $leer$")
+    assert checks.fails_serious(mozillachecker.variables, "Save $file$", "Stoor $leer$")
     assert checks.passes(mozillachecker.variables, "%d files of type %s saved.", "%d leers van %s tipe gestoor.")
-    assert checks.fails(mozillachecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
+    assert checks.fails_serious(mozillachecker.variables, "%d files of type %s saved.", "%s leers van %s tipe gestoor.")
     assert checks.passes(mozillachecker.variables, "Save $file", "Stoor $file")
-    assert checks.fails(mozillachecker.variables, "Save $file", "Stoor $leer")
+    assert checks.fails_serious(mozillachecker.variables, "Save $file", "Stoor $leer")
     assert checks.passes(mozillachecker.variables, "About $ProgramName$", "Oor $ProgramName$")
-    assert checks.fails(mozillachecker.variables, "About $ProgramName$", "Oor $NaamVanProgam$")
+    assert checks.fails_serious(mozillachecker.variables, "About $ProgramName$", "Oor $NaamVanProgam$")
+    assert checks.passes(mozillachecker.variables, "About $_CLICK", "Oor $_CLICK")
+    assert checks.fails_serious(mozillachecker.variables, "About $_CLICK", "Oor $_KLIK")
+    assert checks.passes(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NameDA)")
+    assert checks.fails_serious(mozillachecker.variables, "About $(^NameDA)", "Oor $(^NaamDA)")
     # Double variable problem
-    assert checks.fails(mozillachecker.variables, "Create In &lt;&lt;", "Etsa ka Ho &lt;lt;")
+    assert checks.fails_serious(mozillachecker.variables, "Create In &lt;&lt;", "Etsa ka Ho &lt;lt;")
     # Variables at the end of a sentence
-    assert checks.fails(mozillachecker.variables, "...time you start &brandShortName;.", "...lekgetlo le latelang ha o qala &LebitsoKgutshwane la kgwebo;.")
+    assert checks.fails_serious(mozillachecker.variables, "...time you start &brandShortName;.", "...lekgetlo le latelang ha o qala &LebitsoKgutshwane la kgwebo;.")
     # Ensure that we can detect two variables of the same name with one faulty
-    assert checks.fails(mozillachecker.variables, "&brandShortName; successfully downloaded and installed updates. You will have to restart &brandShortName; to complete the update.", "&brandShortName; ḽo dzhenisa na u longela khwinifhadzo zwavhuḓi. Ni ḓo tea u thoma hafhu &DzinaḼipfufhi ḽa pfungavhuṇe; u itela u fhedzisa khwinifha dzo.")
+    assert checks.fails_serious(mozillachecker.variables, "&brandShortName; successfully downloaded and installed updates. You will have to restart &brandShortName; to complete the update.", "&brandShortName; ḽo dzhenisa na u longela khwinifhadzo zwavhuḓi. Ni ḓo tea u thoma hafhu &DzinaḼipfufhi ḽa pfungavhuṇe; u itela u fhedzisa khwinifha dzo.")
     # We must detect entities in their fullform, ie with fullstop in the middle.
-    assert checks.fails(mozillachecker.variables, "Welcome to the &pluginWizard.title;", "Wamkelekile kwi&Sihloko Soncedo lwe-plugin;")
+    assert checks.fails_serious(mozillachecker.variables, "Welcome to the &pluginWizard.title;", "Wamkelekile kwi&Sihloko Soncedo lwe-plugin;")
     # Variables that are missing in quotes should be detected
-    assert checks.fails(mozillachecker.variables, "\"%S\" is an executable file.... Are you sure you want to launch \"%S\"?", ".... Uyaqiniseka ukuthi ufuna ukuqalisa I\"%S\"?")
+    assert checks.fails_serious(mozillachecker.variables, "\"%S\" is an executable file.... Are you sure you want to launch \"%S\"?", ".... Uyaqiniseka ukuthi ufuna ukuqalisa I\"%S\"?")
     # False positive $ style variables
     assert checks.passes(mozillachecker.variables, "for reporting $ProductShortName$ crash information", "okokubika ukwaziswa kokumosheka kwe-$ProductShortName$")
     # We shouldn't mask variables within variables.  This should highlight &brandShortName as missing and &amp as extra
-    assert checks.fails(mozillachecker.variables, "&brandShortName;", "&amp;brandShortName;")
+    assert checks.fails_serious(mozillachecker.variables, "&brandShortName;", "&amp;brandShortName;")
 
 def test_variables_openoffice():
     """tests variables in OpenOffice translations"""
     # OpenOffice.org variables
     ooochecker = checks.OpenOfficeChecker()
     assert checks.passes(ooochecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandShortname; weergawe.")
-    assert checks.fails(ooochecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandKortnaam; weergawe.")
+    assert checks.fails_serious(ooochecker.variables, "Use the &brandShortname; instance.", "Gebruik die &brandKortnaam; weergawe.")
     assert checks.passes(ooochecker.variables, "Save %file%", "Stoor %file%")
-    assert checks.fails(ooochecker.variables, "Save %file%", "Stoor %leer%")
+    assert checks.fails_serious(ooochecker.variables, "Save %file%", "Stoor %leer%")
     assert checks.passes(ooochecker.variables, "Save %file", "Stoor %file")
-    assert checks.fails(ooochecker.variables, "Save %file", "Stoor %leer")
+    assert checks.fails_serious(ooochecker.variables, "Save %file", "Stoor %leer")
     assert checks.passes(ooochecker.variables, "Save %1", "Stoor %1")
-    assert checks.fails(ooochecker.variables, "Save %1", "Stoor %2")
+    assert checks.fails_serious(ooochecker.variables, "Save %1", "Stoor %2")
     assert checks.passes(ooochecker.variables, "Save %", "Stoor %")
-    assert checks.fails(ooochecker.variables, "Save %", "Stoor")
+    assert checks.fails_serious(ooochecker.variables, "Save %", "Stoor")
     assert checks.passes(ooochecker.variables, "Save $(file)", "Stoor $(file)")
-    assert checks.fails(ooochecker.variables, "Save $(file)", "Stoor $(leer)")
+    assert checks.fails_serious(ooochecker.variables, "Save $(file)", "Stoor $(leer)")
     assert checks.passes(ooochecker.variables, "Save $file$", "Stoor $file$")
-    assert checks.fails(ooochecker.variables, "Save $file$", "Stoor $leer$")
+    assert checks.fails_serious(ooochecker.variables, "Save $file$", "Stoor $leer$")
     assert checks.passes(ooochecker.variables, "Save ${file}", "Stoor ${file}")
-    assert checks.fails(ooochecker.variables, "Save ${file}", "Stoor ${leer}")
+    assert checks.fails_serious(ooochecker.variables, "Save ${file}", "Stoor ${leer}")
     assert checks.passes(ooochecker.variables, "Save #file#", "Stoor #file#")
-    assert checks.fails(ooochecker.variables, "Save #file#", "Stoor #leer#")
+    assert checks.fails_serious(ooochecker.variables, "Save #file#", "Stoor #leer#")
     assert checks.passes(ooochecker.variables, "Save #1", "Stoor #1")
-    assert checks.fails(ooochecker.variables, "Save #1", "Stoor #2")
+    assert checks.fails_serious(ooochecker.variables, "Save #1", "Stoor #2")
     assert checks.passes(ooochecker.variables, "Save #", "Stoor #")
-    assert checks.fails(ooochecker.variables, "Save #", "Stoor")
+    assert checks.fails_serious(ooochecker.variables, "Save #", "Stoor")
     assert checks.passes(ooochecker.variables, "Save ($file)", "Stoor ($file)")
-    assert checks.fails(ooochecker.variables, "Save ($file)", "Stoor ($leer)")
+    assert checks.fails_serious(ooochecker.variables, "Save ($file)", "Stoor ($leer)")
     assert checks.passes(ooochecker.variables, "Save $[file]", "Stoor $[file]")
-    assert checks.fails(ooochecker.variables, "Save $[file]", "Stoor $[leer]")
+    assert checks.fails_serious(ooochecker.variables, "Save $[file]", "Stoor $[leer]")
     assert checks.passes(ooochecker.variables, "Save [file]", "Stoor [file]")
-    assert checks.fails(ooochecker.variables, "Save [file]", "Stoor [leer]")
+    assert checks.fails_serious(ooochecker.variables, "Save [file]", "Stoor [leer]")
     assert checks.passes(ooochecker.variables, "Save $file", "Stoor $file")
-    assert checks.fails(ooochecker.variables, "Save $file", "Stoor $leer")
+    assert checks.fails_serious(ooochecker.variables, "Save $file", "Stoor $leer")
     # Same variable name twice
-    assert checks.fails(ooochecker.variables, r"""Start %PROGRAMNAME% as %PROGRAMNAME%""", "Begin %PROGRAMNAME%")
-    # Variables hidden in KDE comments
-    assert checks.passes(ooochecker.variables, r"""_: Do not translate %PROGRAMNAME% in the text\n
-Start %PRODUCTNAME%""", "Begin %PRODUCTNAME%")
-    # Check how this interacts with the same variable name being repeated
-    assert checks.passes(ooochecker.variables, r"""_: Do not translate %PROGRAMNAME% in the text\n
-Start %PROGRAMNAME%""", "Begin %PROGRAMNAME%")
+    assert checks.fails_serious(ooochecker.variables, r"""Start %PROGRAMNAME% as %PROGRAMNAME%""", "Begin %PROGRAMNAME%")
+
+def test_variables_cclicense():
+    """Tests variables in Creative Commons translations."""
+    checker = checks.CCLicenseChecker()
+    assert checks.passes(checker.variables, "CC-GNU @license_code@.", "CC-GNU @license_code@.")
+    assert checks.fails_serious(checker.variables, "CC-GNU @license_code@.", "CC-GNU @lisensie_kode@.")
+    assert checks.passes(checker.variables, "Deed to the @license_name_full@", "Akte vir die @license_name_full@")
+    assert checks.fails_serious(checker.variables, "Deed to the @license_name_full@", "Akte vir die @volle_lisensie@")
+    assert checks.passes(checker.variables, "The @license_name_full@ is", "Die @license_name_full@ is")
+    assert checks.fails_serious(checker.variables, "The @license_name_full@ is", "Die @iiilicense_name_full@ is")
+    assert checks.fails_serious(checker.variables, "A @ccvar@", "'n @ccvertaaldeveranderlike@")
 
 def test_xmltags():
     """tests xml tags"""
@@ -574,6 +645,10 @@ def test_xmltags():
     assert checks.passes(stdchecker.xmltags, "Click <a xml-lang=\"en\" href=\"page.html\">", "Klik <a xml-lang=\"af\" href=\"page.html\">")
     assert checks.fails(stdchecker.xmltags, "Click <a href=\"page.html\" target=\"koei\">", "Klik <a href=\"page.html\">")
     assert checks.fails(stdchecker.xmltags, "<b>Current Translation</b>", "<b>Traducción Actual:<b>")
+    assert checks.passes(stdchecker.xmltags, "<Error>", "<Fout>")
+    assert checks.fails(stdchecker.xmltags, "%d/%d translated\n(%d blank, %d fuzzy)", "<br>%d/%d μεταφρασμένα\n<br>(%d κενά, %d ασαφή)")
+    frchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="fr"))
+    assert checks.fails(frchecker.xmltags, "Click <a href=\"page.html\">", "Klik <a href=« page.html »>")
 
 def test_ooxmltags():
     """Tests the xml tags in OpenOffice.org translations for quality as done in gsicheck"""
@@ -602,6 +677,8 @@ def test_functions():
     assert checks.passes(stdchecker.functions, "rgb() in percentage", "rgb() kha phesenthe")
     assert checks.fails(stdchecker.functions, "blah string.rgb() blah", "blee bleeb.rgb() blee")
     assert checks.passes(stdchecker.functions, "blah string.rgb() blah", "blee string.rgb() blee")
+    assert checks.passes(stdchecker.functions, "or domain().", "domain() verwag.")
+    assert checks.passes(stdchecker.functions, "Expected url(), url-prefix(), or domain().", "url(), url-prefix() of domain() verwag.")
 
 def test_emails():
     """tests to see that email addresses are not translated"""
@@ -618,5 +695,28 @@ def test_urls():
 def test_simpleplurals():
     """test that we can find English style plural(s)"""
     stdchecker = checks.StandardChecker()
+    assert checks.passes(stdchecker.simpleplurals, "computer(s)", "rekenaar(s)")
     assert checks.fails(stdchecker.simpleplurals, "plural(s)", "meervoud(e)")
     assert checks.fails(stdchecker.simpleplurals, "Ungroup Metafile(s)...", "Kuvhanganyululani Metafaela(dzi)...")
+
+    # Test a language that doesn't use plurals
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='vi'))
+    assert checks.passes(stdchecker.simpleplurals, "computer(s)", u"Máy tính")
+    assert checks.fails(stdchecker.simpleplurals, "computer(s)", u"Máy tính(s)")
+
+def test_nplurals():
+    """test that we can find the wrong number of plural forms"""
+    stdchecker = checks.StandardChecker()
+    assert checks.passes(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d lêer", u"%d lêers"]))
+
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='af'))
+    assert checks.passes(stdchecker.nplurals, "%d files", "%d lêer")
+    assert checks.passes(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d lêer", u"%d lêers"]))
+    assert checks.fails(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d lêer", u"%d lêers", u"%d lêeeeers"]))
+    assert checks.fails(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d lêer"]))
+
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='km'))
+    assert checks.passes(stdchecker.nplurals, "%d files", "%d ឯកសារ")
+    assert checks.passes(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d ឯកសារ"]))
+    assert checks.fails(stdchecker.nplurals, multistring(["%d file", "%d files"]), multistring([u"%d ឯកសារ", u"%d lêers"]))
+
