@@ -21,11 +21,12 @@
 
 """Class to perform translation memory matching from a store of translation units"""
 
-import Levenshtein
-import terminology
-import heapq
+from translate.search import lshtein
+from translate.search import terminology
 from translate.storage import base
 from translate.storage import po
+from translate.misc.multistring import multistring
+import heapq
 
 def sourcelen(unit):
     """Returns the length of the source string"""
@@ -45,7 +46,7 @@ class matcher:
         min_similarity is the minimum similarity that must be attained to be included in
         the result, comparer is an optional Comparer with similarity() function"""
         if comparer is None:
-            comparer = Levenshtein.LevenshteinComparer(max_length)
+            comparer = lshtein.LevenshteinComparer(max_length)
         self.comparer = comparer
         self.setparameters(max_candidates, min_similarity, max_length)
         self.inittm(store)
@@ -64,26 +65,54 @@ class matcher:
                 return True
         return False
 
-    def inittm(self, store):
+    def inittm(self, stores):
         """Initialises the memory for later use. We use simple base units for 
         speedup."""
         self.existingunits = {}
         self.candidates = base.TranslationStore()
         
-        if not isinstance(store, list):
-            store = [store]
-        for file in store:
-            candidates = filter(self.usable, file.units)
-            for candidate in candidates:
-                simpleunit = base.TranslationUnit(candidate.source)
-                simpleunit.target = candidate.target
-                simpleunit.addnote(candidate.getnotes())
-                self.candidates.units.append(simpleunit)
+        if not isinstance(stores, list):
+            stores = [stores]
+        for store in stores:
+            self.extendtm(store.units, store=store, sort=False)
         self.candidates.units.sort(sourcelencmp)
-        if not self.candidates.units:
-            raise ValueError("No usable translation memory")
-        print "TM initialised with %d candidates (%d to %d characters long)" % \
-                (len(self.candidates.units), len(self.candidates.units[0].source), len(self.candidates.units[-1].source))
+        # print "TM initialised with %d candidates (%d to %d characters long)" % \
+        #        (len(self.candidates.units), len(self.candidates.units[0].source), len(self.candidates.units[-1].source))
+
+    def extendtm(self, units, store=None, sort=True):
+        """Extends the memory with extra unit(s).
+        
+        @param units: The units to add to the TM.
+        @param store: Optional store from where some metadata can be retrieved
+        and associated with each unit.
+        @param sort:  Optional parameter that can be set to False to supress 
+        sorting of the candidates list. This should probably only be used in 
+        inittm().
+        """
+        if not isinstance(units, list):
+            units = [units]
+        candidates = filter(self.usable, units)
+        for candidate in candidates:
+            simpleunit = base.TranslationUnit("")
+            # We need to ensure that we don't pass multistrings futher, since
+            # some modules (like the native Levenshtein) can't use it.
+            if isinstance(candidate.source, multistring):
+                if len(candidate.source.strings) > 1:
+                    simpleunit.orig_source = candidate.source
+                    simpleunit.orig_target = candidate.target
+                simpleunit.source = unicode(candidate.source)
+                simpleunit.target = unicode(candidate.target)
+            else:
+                simpleunit.source = candidate.source
+                simpleunit.target = candidate.target
+            simpleunit.addnote(candidate.getnotes(origin="translator"))
+            if store:
+                simpleunit.filepath = store.filepath
+                simpleunit.translator = store.translator
+                simpleunit.date = store.date
+            self.candidates.units.append(simpleunit)
+        if sort:
+            self.candidates.units.sort(sourcelencmp)
 
     def setparameters(self, max_candidates=10, min_similarity=75, max_length=70):
         """Sets the parameters without reinitialising the tm. If a parameter 
@@ -103,8 +132,15 @@ class matcher:
         return max(len(text) * (min_similarity/100.0), 1)
     
     def matches(self, text):
-        """Returns a list of possible matches for text in candidates with the associated similarity.
-        Return value is a list containing tuples (score, original, translation)."""
+        """Returns a list of possible matches for given source text.
+        
+        @type text: String
+        @param text: The text that will be search for in the translation memory
+        @rtype: list
+        @return: a list of units with the source and target strings from the 
+        translation memory. If self.addpercentage is true (default) the match 
+        quality is given as a percentage in the notes.
+        """
         bestcandidates = [(0.0,None)]*self.MAX_CANDIDATES
         heapq.heapify(bestcandidates)
         #We use self.MIN_SIMILARITY, but if we already know we have max_candidates
@@ -154,8 +190,14 @@ class matcher:
         """Builds a list of units conforming to base API, with the score in the comment"""
         units = []
         for score, candidate in candidates:
+            if hasattr(candidate, "orig_source"):
+                candidate.source = candidate.orig_source
+                candidate.target = candidate.orig_target
             newunit = po.pounit(candidate.source)
             newunit.target = candidate.target
+            newunit.filepath = candidate.filepath
+            newunit.translator = candidate.translator
+            newunit.date = candidate.date
             candidatenotes = candidate.getnotes().strip()
             if candidatenotes:
                 newunit.addnote(candidatenotes)
