@@ -49,7 +49,7 @@ class SimpleDictReader:
   def getvalue(self, value):
     """returns a value, evaluating strings as neccessary"""
     if (value.startswith("'") and value.endswith("'")) or (value.startswith('"') and value.endswith('"')):
-      return sparse.stringeval(value).replace("\r","").replace("\n","")
+      return sparse.stringeval(value)
     else:
       return value
 
@@ -96,6 +96,7 @@ class SimpleDictReader:
 class csvunit(base.TranslationUnit):
   spreadsheetescapes = [("+", "\\+"), ("-", "\\-"), ("=", "\\="), ("'", "\\'")]
   def __init__(self, source=None):
+    super(csvunit, self).__init__(source)
     self.comment = ""
     self.source = source
     self.target = ""
@@ -143,6 +144,7 @@ class csvfile(base.TranslationStore):
   The default format contains three columns: comments, source, target"""
   UnitClass = csvunit
   def __init__(self, inputfile=None, fieldnames=None):
+    base.TranslationStore.__init__(self, unitclass = self.UnitClass)
     self.units= []
     if fieldnames is None:
       self.fieldnames = ['comment', 'source', 'target']
