@@ -30,19 +30,44 @@ from translate.storage import tmx
 from translate.storage import tbx
 
 import os
+from gzip import GzipFile
+from bz2 import BZ2File
 
 #TODO: Monolingual formats (with template?)
-#TODO: guess from first lines
 
 classes = {"po": po.pofile, "pot": po.pofile, "csv": csvl10n.csvfile, 
             "xliff": xliff.xlifffile, "xlf": xliff.xlifffile, 
             "tmx": tmx.tmxfile, "tbx": tbx.tbxfile}
 
+decompressclass = {
+    'gz': GzipFile,
+    'bz2': BZ2File,
+}
+
+def guessextention(storefile):
+    """Guesses the type of a file object by looking at the first few characters.
+    The return value is a file extention ."""
+    start = storefile.read(200).strip()
+    if '<xliff ' in start:
+        extention = 'xlf'
+    elif 'msgid "' in start:
+        extention = 'po'
+    else:
+        raise ValueError("Failed to guess file type.")
+    storefile.seek(0)
+    return extention
+
+def getdummyname(storefile):
+    """Provides a dummy name for a file object without a name attribute, by guessing the file type."""
+    return 'dummy.' + guessextention(storefile)
+
 def getname(storefile):
     """returns the filename"""
+    if storefile is None:
+        raise ValueError("This method cannot magically produce a filename when given None as input.")
     if not isinstance(storefile, basestring):
         if not hasattr(storefile, "name"):
-            raise ValueError("Factory can only guess filetype from filename")
+            storefilename = getdummyname(storefile)
         else:
             storefilename = storefile.name
     else:
@@ -57,20 +82,37 @@ def getclass(storefile, ignore=None):
         storefilename = storefile[:-len(ignore)]
     root, ext = os.path.splitext(storefilename)
     ext = ext[len(os.path.extsep):].lower()
+    if ext in decompressclass:
+        root, ext = os.path.splitext(root)
+        ext = ext[len(os.path.extsep):].lower()
     try:
         storeclass = classes[ext]
     except KeyError:
-        raise Exception("Unknown filetype (%s)" % storefilename)
+        raise ValueError("Unknown filetype (%s)" % storefilename)
     return storeclass
 
 def getobject(storefile, ignore=None):
     """Factory that returns a usable object for the type of file presented.
-    Specify ignore to ignore some part at the back of the name (like .gz). """
+
+    @type storefile: file or str
+    @param storefile: File object or file name.
+
+    Specify ignore to ignore some part at the back of the name (like .gz).
+    """
+
     if isinstance(storefile, base.TranslationStore):
         return storefile
+    if isinstance(storefile, basestring):
+        if os.path.isdir(storefile) or storefile.endswith(os.path.sep):
+            from translate.storage import directory
+            return directory.Directory(storefile)
     storefilename = getname(storefile)
     storeclass = getclass(storefilename, ignore)
     if os.path.exists(storefilename) or not getattr(storefile, "closed", True):
+        name, ext = os.path.splitext(storefilename)
+        ext = ext[len(os.path.extsep):].lower()
+        if ext in decompressclass:
+            storefile = decompressclass[ext](storefilename)
         store = storeclass.parsefile(storefile)
     else:
         store = storeclass()
