@@ -95,7 +95,8 @@ class ooline:
 
   def getoutput(self):
     """return a line in tab-delimited form"""
-    return "\t".join(self.getparts()).replace("\n", "\\n")
+    parts = [part.replace("\n", "\\n").replace("\t", "\\t").replace("\r", "\\r") for part in self.getparts()]
+    return "\t".join(parts)
 
   def getkey(self):
     """get the key that identifies the resource"""
@@ -132,7 +133,7 @@ class oofile:
     self.oolines = []
     self.units = []
     self.ookeys = {}
-    self.filename = "(unknown file)"
+    self.filename = ""
     self.languages = []
     if input is not None:
       self.parse(input)
@@ -152,7 +153,8 @@ class oofile:
 
   def parse(self, input):
     """parses lines and adds them to the file"""
-    self.filename = getattr(input, 'name', '')
+    if not self.filename:
+        self.filename = getattr(input, 'name', '')
     if hasattr(input, "read"):
       src = input.read()
       input.close()
