@@ -25,11 +25,10 @@ gettext-style .po (or .pot) files are used in translations for KDE et al (see kb
 from __future__ import generators
 from translate.misc.multistring import multistring
 from translate.misc import quote
-from translate.misc import dictutils
+from translate.misc import textwrap
 from translate.storage import base
-from translate import __version__
-import sre
-import time
+from translate.storage import poheader
+import re
 import codecs
 
 # general functions for quoting / unquoting po strings
@@ -38,7 +37,10 @@ po_unescape_map = {"\\r": "\r", "\\t": "\t", '\\"': '"', '\\n': '\n', '\\\\': '\
 po_escape_map = dict([(value, key) for (key, value) in po_unescape_map.items()])
 
 def escapeforpo(line):
-  """escapes a line for po format. assumes no \n occurs in the line"""
+  """Escapes a line for po format. assumes no \n occurs in the line.
+  
+  @param line: unescaped text
+  """
   special_locations = []
   for special_key in po_escape_map:
     special_locations.extend(quote.find_all(line, special_key))
@@ -54,72 +56,51 @@ def escapeforpo(line):
   return escaped_line
 
 def unescapehandler(escape):
+
   return po_unescape_map.get(escape, escape)
 
-def quoteforpo(text, template=None):
+def wrapline(line):
+    """Wrap text for po files."""
+    wrappedlines = textwrap.wrap(line, 76, replace_whitespace=False, expand_tabs=False, drop_whitespace=False)
+
+    # Lines should not start with a space...
+    if len(wrappedlines) > 1:
+        for index, line in enumerate(wrappedlines[1:]):
+            if line.startswith(' '):
+                # Remove the space at the beginning of the line:
+                wrappedlines[index+1] = line[1:]
+
+                # Append a space to the previous line:
+                wrappedlines[index] += ' '
+    return wrappedlines
+
+def quoteforpo(text):
   """quotes the given text for a PO file, returning quoted and escaped lines"""
-  if template:
-    return quoteforpofromtemplate(text, template)
   polines = []
   if text is None:
     return polines
   lines = text.split("\n")
-  if len(lines) > 1:
-    polines.extend(['""'])
-    polines.extend(['"' + escapeforpo(line) + '\\n"' for line in lines[:-1]])
+  if len(lines) > 1 or (len(lines) == 1 and len(lines[0]) > 71):
+    if len(lines) != 2 or lines[1]:
+        polines.extend(['""'])
+    for line in lines[:-1]:
+      lns = wrapline(line)
+      if len(lns) > 0:
+        for ln in lns[:-1]:
+          polines.extend(['"' + escapeforpo(ln) + '"'])
+        if lns[-1]:
+          polines.extend(['"' + escapeforpo(lns[-1]) + '\\n"'])
+      else:
+        polines.extend(['"\\n"'])
   if lines[-1]:
-    polines.extend(['"' + escapeforpo(lines[-1]) + '"'])
-  return polines
-
-def quoteforpofromtemplate(text, template):
-  """Same as quoteforpo, but try to to use same format as template as far as
-  possible. template is a list of polines (such as pounit.msgstr)"""
-  # TODO: check whether this may alter the underlying po.msgid
-  for position, item in enumerate(template):
-    if not isinstance(item, unicode):
-      template[position] = item.decode('utf-8')
-
-  templatetext = unquotefrompo(template)
-  if templatetext and len(template) == 1:
-    return quoteforpo(text)
-
-  #unchanged is a list containing tuples indicating
-  #    (start in text, end in text, quoted part)
-  unchanged = []
-  index = -1
-  searchfrom = 0
-  for part in template:
-    unquotedpart = unquotefrompo([part])
-    if len(unquotedpart) == 0:
-      continue
-    index = text.find(unquotedpart, searchfrom)
-    if index >= 0:
-      searchfrom = index + len(unquotedpart)
-      unchanged.append((index, searchfrom, part))
-
-  #index indicates up to where in text we have processed:
-  index = 0
-  polines = []
-  while index < len(text):
-    if len(unchanged) == 0:
-      polines.extend(quoteforpo(text[index:]))
-      index = len(text)
-    else:
-      (start, end, part) = unchanged.pop(0)
-      if index < start:
-        polines.extend(quoteforpo(text[index:start]))
-      polines.append(part)
-      index = end
-
-  #Some fixups with empty ("") lines
-  if len(template) > 1 and template[0] == '""':
-    polines = ['""'] + polines
-  elif polines and polines[0] == '""':
-      polines = polines[1:]
-    
+    polines.extend(['"' + escapeforpo(line) + '"' for line in wrapline(lines[-1])])
   return polines
 
 def extractpoline(line):
+  """Remove quote and unescape line from po file.
+   
+  @param line: a quoted line from a po file (msgid or msgstr)
+  """
   extracted = quote.extractwithoutquotes(line,'"','"','\\',includeescapes=unescapehandler)[0]
   return extracted
 
@@ -149,8 +130,10 @@ From the GNU gettext manual:
      WHITE-SPACE
      #  TRANSLATOR-COMMENTS
      #. AUTOMATIC-COMMENTS
+     #| PREVIOUS MSGID                 (Gettext 0.16 - check if this is the correct position - not yet implemented)
      #: REFERENCE...
      #, FLAG...
+     msgctxt CONTEXT                   (Gettext 0.15)
      msgid UNTRANSLATED-STRING
      msgstr TRANSLATED-STRING
 """
@@ -162,6 +145,7 @@ class pounit(base.TranslationUnit):
   # typecomments = []       #   #, fuzzy
   # visiblecomments = []    #   #_ note to translator  (this is nonsense)
   # msgidcomments = []      #   _: within msgid
+  # msgctxt
   # msgid = []
   # msgstr = []
 
@@ -169,16 +153,19 @@ class pounit(base.TranslationUnit):
     self.encoding = encodingToUse(encoding)
     self.obsolete = False
     self.initallcomments(blankall=True)
+    self.msgctxt = []
     self.msgid = []
     self.msgid_pluralcomments = []
     self.msgid_plural = []
     self.msgstr = []
+    self.obsoletemsgctxt = []
     self.obsoletemsgid = []
     self.obsoletemsgid_pluralcomments = []
     self.obsoletemsgid_plural = []
     self.obsoletemsgstr = []
     if source:
       self.setsource(source)
+    super(pounit, self).__init__(source)
 
   def initallcomments(self, blankall=False):
     """Initialises allcomments"""
@@ -202,19 +189,27 @@ class pounit(base.TranslationUnit):
     """Returns the unescaped msgid"""
     multi = multistring(unquotefrompo(self.msgid), self.encoding)
     if self.hasplural():
-      multi.strings.append(unquotefrompo(self.msgid_plural))
+      pluralform = unquotefrompo(self.msgid_plural)
+      if isinstance(pluralform, str):
+          pluralform = pluralform.decode(self.encoding)
+      multi.strings.append(pluralform)
     return multi
 
   def setsource(self, source):
-    """Sets the msgstr to the given (unescaped) value"""
+    """Sets the msgid to the given (unescaped) value.
+    
+    @param source: an unescaped source string.
+    """
+    if isinstance(source, str):
+      source = source.decode(self.encoding)
     if isinstance(source, multistring):
       source = source.strings
     if isinstance(source, list):
       self.msgid = quoteforpo(source[0])
       if len(source) > 1:
-        self.msgid_plural = quoteforpo(source[1], self.msgid_plural)
+        self.msgid_plural = quoteforpo(source[1])
     else:
-      self.msgid = quoteforpo(source, self.msgid)
+      self.msgid = quoteforpo(source)
   source = property(getsource, setsource)
 
   def gettarget(self):
@@ -223,33 +218,54 @@ class pounit(base.TranslationUnit):
       multi = multistring(map(unquotefrompo, self.msgstr.values()), self.encoding)
     else:
       multi = multistring(unquotefrompo(self.msgstr), self.encoding)
-    return multi.replace("\\n", "\n").replace("\\t", "\t").replace("\\r", "\r")
+    return multi
 
   def settarget(self, target):
     """Sets the msgstr to the given (unescaped) value"""
+    if isinstance(target, str):
+      target = target.decode(self.encoding)
     if target == self.target:
       return
-    if isinstance(target, multistring) and len(target.strings) > 1:
-      target = target.strings
+    if self.hasplural():
+      if isinstance(target, multistring):
+        target = target.strings
+      elif isinstance(target, basestring):
+        target = [target]
+    elif isinstance(target,(dict, list)):
+      if len(target) == 1:
+        target = target[0]
+      else:
+        raise ValueError("po msgid element has no plural but msgstr has %d elements (%s)" % (len(target), target))
     templates = self.msgstr
     if isinstance(templates, list):
       templates = {0: templates}
     if isinstance(target, list):
-      self.msgstr = dict([(i, quoteforpo(target[i], templates.get(i, None))) for i in range(len(target))])
+      self.msgstr = dict([(i, quoteforpo(target[i])) for i in range(len(target))])
     elif isinstance(target, dict):
-      self.msgstr = dict([(i, quoteforpo(targetstring, templates.get(i, None))) for i, targetstring in target.iteritems()])
+      self.msgstr = dict([(i, quoteforpo(targetstring)) for i, targetstring in target.iteritems()])
     else:
-      self.msgstr = quoteforpo(target, template=templates.get(0, None))
+      self.msgstr = quoteforpo(target)
   target = property(gettarget, settarget)
 
-  def getnotes(self):
-    comments = "".join([comment[2:] for comment in self.othercomments])
-    comments += "".join([comment[3:] for comment in self.automaticcomments])
+  def getnotes(self, origin=None):
+    """Return comments based on origin value (programmer, developer, source code and translator)"""
+    if origin == None:
+      comments = "".join([comment[2:] for comment in self.othercomments])
+      comments += "".join([comment[3:] for comment in self.automaticcomments])
+    elif origin == "translator":
+      comments = "".join ([comment[2:] for comment in self.othercomments])
+    elif origin in ["programmer", "developer", "source code"]:
+      comments = "".join([comment[3:] for comment in self.automaticcomments])
+    else:
+      raise ValueError("Comment type not valid")
     # Let's drop the last newline
     return comments[:-1]
 
   def addnote(self, text, origin=None):
     """This is modeled on the XLIFF method. See xliff.py::xliffunit.addnote"""
+    # We don't want to put in an empty '#' without a real comment:
+    if not text:
+      return
     commentlist = self.othercomments
     linestart = "# "
     if origin in ["programmer", "developer", "source code"]:
@@ -258,6 +274,28 @@ class pounit(base.TranslationUnit):
     text = text.split("\n")
     commentlist += [linestart + line + "\n" for line in text]
     
+  def removenotes(self):
+    """Remove all the translator's notes (other comments)"""
+    self.othercomments = []
+
+  def adderror(self, errorname, errortext):
+    """Adds an error message to this unit."""
+    text = u'(pofilter) %s: %s' % (errorname, errortext)
+    # Don't add the same error twice:
+    if text not in self.getnotes(origin='translator'):
+        self.addnote(text, origin="translator")
+
+  def geterrors(self):
+    """Get all error messages."""
+    notes = self.getnotes(origin="translator").split('\n')
+    errordict = {}
+    for note in notes:
+      if '(pofilter) ' in note:
+        error = note.replace('(pofilter) ', '')
+        errorname, errortext = error.split(': ')
+        errordict[errorname] = errortext
+    return errordict
+
   def copy(self):
     newpo = self.__class__()
     newpo.othercomments = self.othercomments[:]
@@ -268,6 +306,7 @@ class pounit(base.TranslationUnit):
     newpo.obsolete = self.obsolete
     newpo.msgidcomments = self.msgidcomments[:]
     newpo.initallcomments()
+    newpo.msgctxt = self.msgctxt[:]
     newpo.msgid = self.msgid[:]
     newpo.msgid_pluralcomments = self.msgid_pluralcomments[:]
     newpo.msgid_plural = self.msgid_plural[:]
@@ -276,6 +315,7 @@ class pounit(base.TranslationUnit):
     else:
       newpo.msgstr = self.msgstr[:]
       
+    newpo.obsoletemsgctxt = self.obsoletemsgctxt[:]
     newpo.obsoletemsgid = self.obsoletemsgid[:]
     newpo.obsoletemsgid_pluralcomments = self.obsoletemsgid_pluralcomments[:]
     newpo.obsoletemsgid_plural = self.obsoletemsgid_plural[:]
@@ -298,16 +338,14 @@ class pounit(base.TranslationUnit):
     else:
       return len(unquotefrompo(self.msgstr).strip())
 
-  def differentmsgidcomments(self, othercomments):
-    """merges #. automatic comments only if necessary"""
-    mycomment = unquotefrompo(self.msgidcomments)[2:]
-    othercomment = unquotefrompo(othercomments)[2:]
-    return mycomment != othercomment
+  def merge(self, otherpo, overwrite=False, comments=True, authoritative=False):
+    """Merges the otherpo (with the same msgid) into this one.
+
+    Overwrite non-blank self.msgstr only if overwrite is True
+    merge comments only if comments is True
+    
+    """
 
-  def merge(self, otherpo, overwrite=False, comments=True):
-    """merges the otherpo (with the same msgid) into this one
-    overwrite non-blank self.msgstr only if overwrite is True
-    merge comments only if comments is True"""
     def mergelists(list1, list2, split=False):
       #decode where necessary
       if unicode in [type(item) for item in list2] + [type(item) for item in list1]:
@@ -353,20 +391,23 @@ class pounit(base.TranslationUnit):
       return
     if comments:
       mergelists(self.othercomments, otherpo.othercomments)
-      #We don't bring acros otherpo.automaticcomments as we consider ourself
-      #to be the authority.
-      #self.automaticcomments = otherpo.automaticcomments
-      mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
       mergelists(self.typecomments, otherpo.typecomments)
       mergelists(self.visiblecomments, otherpo.visiblecomments)
-      if self.differentmsgidcomments(otherpo.msgidcomments):
+      if not authoritative:
+        # We don't bring across otherpo.automaticcomments as we consider ourself
+        # to be the the authority.  Same applies to otherpo.msgidcomments
+        mergelists(self.automaticcomments, otherpo.automaticcomments)
         mergelists(self.msgidcomments, otherpo.msgidcomments)
-      else:
-        self.msgidcomments = otherpo.msgidcomments
+        mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
     if self.isblankmsgstr() or overwrite:
+      # Remove kde-style comments from the translation (if any).
+      if self.extract_msgidcomments(otherpo.target):
+        otherpo.target = otherpo.target.replace('_: ' + otherpo.extract_msgidcomments()+ '\n', '')
       self.target = otherpo.target
       if self.source != otherpo.source:
         self.markfuzzy()
+      else:
+        self.markfuzzy(otherpo.isfuzzy())
     elif otherpo.isblankmsgstr():
       if self.source != otherpo.source:
         self.markfuzzy()
@@ -375,15 +416,21 @@ class pounit(base.TranslationUnit):
         self.markfuzzy()
 
   def isheader(self):
-    return (self.msgidlen() == 0) and (self.msgstrlen() > 0) and (len(self.msgidcomments) == 0)
+    #return (self.msgidlen() == 0) and (self.msgstrlen() > 0) and (len(self.msgidcomments) == 0)
+    #rewritten here for performance:
+    return ((self.msgid == [] or self.msgid == ['""']) and 
+            not (self.msgstr == [] or self.msgstr == ['""']) 
+            and self.msgidcomments == [])
 
   def isblank(self):
-    if self.isheader():
+    if self.isheader() or len(self.msgidcomments):
       return False
     if (self.msgidlen() == 0) and (self.msgstrlen() == 0):
       return True
-    unquotedid = [quote.extractwithoutquotes(line,'"','"','\\',includeescapes=0)[0] for line in self.msgid]
-    return len("".join(unquotedid).strip()) == 0
+    return False
+    # TODO: remove:
+    # Before, the equivalent of the following was the final return statement:
+    # return len(self.source.strip()) == 0
 
   def isblankmsgstr(self):
     """checks whether the msgstr is blank"""
@@ -392,7 +439,7 @@ class pounit(base.TranslationUnit):
   def hastypecomment(self, typecomment):
     """check whether the given type comment is present"""
     # check for word boundaries properly by using a regular expression...
-    return sum(map(lambda tcline: len(sre.findall("\\b%s\\b" % typecomment, tcline)), self.typecomments)) != 0
+    return sum(map(lambda tcline: len(re.findall("\\b%s\\b" % typecomment, tcline)), self.typecomments)) != 0
 
   def hasmarkedcomment(self, commentmarker):
     """check whether the given comment marker is present as # (commentmarker) ..."""
@@ -409,9 +456,12 @@ class pounit(base.TranslationUnit):
         self.typecomments.append("#, %s\n" % typecomment)
       else:
         # this should handle word boundaries properly ...
-        typecomments = map(lambda tcline: sre.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline), self.typecomments)
+        typecomments = map(lambda tcline: re.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline), self.typecomments)
         self.typecomments = filter(lambda tcline: tcline.strip() != "#,", typecomments)
 
+  def istranslated(self):
+    return super(pounit, self).istranslated() and not self.isobsolete()
+
   def isfuzzy(self):
     return self.hastypecomment("fuzzy")
 
@@ -421,6 +471,26 @@ class pounit(base.TranslationUnit):
   def isreview(self):
     return self.hastypecomment("review") or self.hasmarkedcomment("review") or self.hasmarkedcomment("pofilter")
 
+  def markreviewneeded(self, needsreview=True, explanation=None):
+    """Marks the unit to indicate whether it needs review. Adds an optional explanation as a note."""
+    if needsreview:
+      reviewnote = "(review)"
+      if explanation:
+        reviewnote += " " + explanation
+      self.addnote(reviewnote, origin="translator")
+    else:
+      # Strip (review) notes.
+      notestring = self.getnotes(origin="translator")
+      notes = notestring.split('\n')
+      newnotes = []
+      for note in notes:
+        if not '(review)' in note:
+          newnotes.append(note)
+      newnotes = '\n'.join(newnotes)
+      self.removenotes()
+      self.addnote(newnotes, origin="translator")
+      
+
   def isnotblank(self):
     return not self.isblank()
 
@@ -430,6 +500,8 @@ class pounit(base.TranslationUnit):
   def makeobsolete(self):
     """Makes this unit obsolete"""
     self.obsolete = True
+    if self.msgctxt:
+      self.obsoletemsgctxt = self.msgctxt
     if self.msgid:
       self.obsoletemsgid = self.msgid
       self.msgid = []
@@ -448,6 +520,9 @@ class pounit(base.TranslationUnit):
   def resurrect(self):
     """Makes an obsolete unit normal"""
     self.obsolete = False
+    if self.obsoletemsgctxt:
+      self.msgid = self.obsoletemsgctxt
+      self.obsoletemsgctxt = []
     if self.obsoletemsgid:
       self.msgid = self.obsoletemsgid
       self.obsoletemsgid = []
@@ -466,6 +541,7 @@ class pounit(base.TranslationUnit):
     return len(self.msgid_plural) > 0
 
   def parse(self, src):
+    inmsgctxt = 0
     inmsgid = 0
     inmsgid_comment = 0
     inmsgid_plural = 0
@@ -495,16 +571,25 @@ class pounit(base.TranslationUnit):
         else:
           self.othercomments.append(line)
       if line.startswith('msgid_plural'):
+        inmsgctxt = 0
         inmsgid = 0
         inmsgid_plural = 1
         inmsgstr = 0
         inmsgid_comment = 0
+      elif line.startswith('msgctxt'):
+        inmsgctxt = 1
+        inmsgid = 0
+        inmsgid_plural = 0
+        inmsgstr = 0
+        inmsgid_comment = 0
       elif line.startswith('msgid'):
+        inmsgctxt = 0
         inmsgid = 1
         inmsgid_plural = 0
         inmsgstr = 0
         inmsgid_comment = 0
       elif line.startswith('msgstr'):
+        inmsgctxt = 0
         inmsgid = 0
         inmsgid_plural = 0
         inmsgstr = 1
@@ -514,7 +599,9 @@ class pounit(base.TranslationUnit):
           msgstr_pluralid = None
       extracted = quote.extractstr(line)
       if not extracted is None:
-        if inmsgid:
+        if inmsgctxt:
+          self.msgctxt.append(extracted)
+        elif inmsgid:
           # TODO: improve kde comment detection
           if extracted.find("_:") != -1:
             inmsgid_comment = 1
@@ -544,6 +631,12 @@ class pounit(base.TranslationUnit):
             self.msgstr[msgstr_pluralid].append(extracted)
     if self.obsolete:
       self.makeobsolete()
+    # If this unit is the header, we have to get the encoding to ensure that no
+    # methods are called that need the encoding before we obtained it.
+    if self.isheader():
+      charset = re.search("charset=([^\\s]+)", unquotefrompo(self.msgstr))
+      if charset:
+        self.encoding = encodingToUse(charset.group(1))
     return linesprocessed
 
   def getmsgpartstr(self, partname, partlines, partcomments=""):
@@ -577,7 +670,7 @@ class pounit(base.TranslationUnit):
             comment = comment[:-len("\\n")]
           #Before we used to strip. Necessary in some cases?
           combinedcomment.append(comment)
-        partcomments = quoteforpo("_:%s" % "".join(combinedcomment), partcomments)
+        partcomments = quoteforpo("_:%s" % "".join(combinedcomment))
       # comments first, no blank leader line needed
       partstr += "\n".join(partcomments)
       partstr = quote.rstripeol(partstr)
@@ -608,6 +701,8 @@ class pounit(base.TranslationUnit):
     if self.isobsolete():
       lines.extend(self.typecomments)
       obsoletelines = []
+      if self.obsoletemsgctxt:
+        obsoletelines.append(self.getmsgpartstr("#~ msgctxt", self.obsoletemsgctxt))
       obsoletelines.append(self.getmsgpartstr("#~ msgid", self.obsoletemsgid, self.obsoletemsgidcomments))
       if self.obsoletemsgid_plural or self.obsoletemsgid_pluralcomments:
         obsoletelines.append(self.getmsgpartstr("#~ msgid_plural", self.obsoletemsgid_plural, self.obsoletemsgid_pluralcomments))
@@ -627,6 +722,8 @@ class pounit(base.TranslationUnit):
     lines.extend(self.sourcecomments)
     lines.extend(self.typecomments)
     lines.extend(self.visiblecomments)
+    if self.msgctxt:
+      lines.append(self.getmsgpartstr("msgctxt", self.msgctxt))
     lines.append(self.getmsgpartstr("msgid", self.msgid, self.msgidcomments))
     if self.msgid_plural or self.msgid_pluralcomments:
       lines.append(self.getmsgpartstr("msgid_plural", self.msgid_plural, self.msgid_pluralcomments))
@@ -636,185 +733,70 @@ class pounit(base.TranslationUnit):
     return postr
 
   def getlocations(self):
-    """returns the list of locations from sourcecomments in the po element"""
+    """Get a list of locations from sourcecomments in the PO unit
+
+    rtype: List
+    return: A list of the locations with '#: ' stripped
+
+    """
     locations = []
     for sourcecomment in self.sourcecomments:
       locations += quote.rstripeol(sourcecomment)[3:].split()
     return locations
 
   def addlocation(self, location):
-    """add to sourcecomments"""
+    """Add a location to sourcecomments in the PO unit
+
+    @param location: Text location e.g. 'file.c:23' does not include #:
+    @type location: String
+
+    """
     self.sourcecomments.append("#: %s\n" % location)
 
-class pofile(base.TranslationStore):
+  def extract_msgidcomments(self, text=None):
+    """Extract KDE style msgid comments from the unit.
+    
+    @rtype: String
+    @return: Returns the extracted msgidcomments found in this unit's msgid.
+    
+    """
+
+    if not text:
+        text = unquotefrompo(self.msgidcomments)
+    return text.split('\n')[0].replace('_: ', '', 1)
+
+  def getcontext(self):
+    """Get the message context."""
+    return unquotefrompo(self.msgctxt) + self.extract_msgidcomments()
+
+class pofile(base.TranslationStore, poheader.poheader):
   """this represents a .po file containing various units"""
   UnitClass = pounit
-  x_generator = "Translate Toolkit %s" % __version__.ver
-  header_order = [
-    "Project-Id-Version",
-    "Report-Msgid-Bugs-To",
-    "POT-Creation-Date",
-    "PO-Revision-Date",
-    "Last-Translator",
-    "Language-Team",
-    "MIME-Version",
-    "Content-Type",
-    "Content-Transfer-Encoding",
-    "Plural-Forms",
-    "X-Generator",
-    ]
   def __init__(self, inputfile=None, encoding=None, unitclass=pounit):
     """construct a pofile, optionally reading in from inputfile.
     encoding can be specified but otherwise will be read from the PO header"""
     self.UnitClass = unitclass
+    base.TranslationStore.__init__(self, unitclass=unitclass)
     self.units = []
     self.filename = ''
     self.encoding = encodingToUse(encoding)
     if inputfile is not None:
       self.parse(inputfile)
 
-  def header(self):
-    """Returns the header element, or None. Only the first element is allowed
-    to be a header. Note that this could still return an empty header element,
-    if present."""
-    if len(self.units) == 0:
-      return None
-    candidate = self.units[0]
-    if candidate.isheader():
-      return candidate
-    else:
-      return None
-
-  def makeheader(self, charset="CHARSET", encoding="ENCODING", project_id_version=None, pot_creation_date=None, po_revision_date=None, last_translator=None, language_team=None, mime_version=None, plural_forms=None, report_msgid_bugs_to=None, **kwargs):
+  def makeheader(self, **kwargs):
     """create a header for the given filename. arguments are specially handled, kwargs added as key: value
     pot_creation_date can be None (current date) or a value (datetime or string)
     po_revision_date can be None (form), False (=pot_creation_date), True (=now), or a value (datetime or string)"""
-    headerargs = dictutils.cidict()
-    for key, value in kwargs.iteritems():
-      key = key.replace("_", "-")
-      if key.islower():
-        key = key.title()
-      headerargs[key] = value
+
     headerpo = self.UnitClass(encoding=self.encoding)
     headerpo.markfuzzy()
     headerpo.msgid = ['""']
-    headeritems = [""]
-    if project_id_version is None:
-      project_id_version = "PACKAGE VERSION"
-    if pot_creation_date is None or pot_creation_date == True:
-      pot_creation_date = time.strftime("%Y-%m-%d %H:%M%z")
-    if isinstance(pot_creation_date, time.struct_time):
-      pot_creation_date = pot_creation_date.strftime("%Y-%m-%d %H:%M%z")
-    if po_revision_date is None:
-      po_revision_date = "YEAR-MO-DA HO:MI+ZONE"
-    elif po_revision_date == False:
-      po_revision_date = pot_creation_date
-    elif po_revision_date == True:
-      po_revision_date = time.strftime("%Y-%m-%d %H:%M%z")
-    if isinstance(po_revision_date, time.struct_time):
-      po_revision_date = po_revision_date.strftime("%Y-%m-%d %H:%M%z")
-    if last_translator is None:
-      last_translator = "FULL NAME <EMAIL@ADDRESS>"
-    if language_team is None:
-      language_team = "LANGUAGE <LL@li.org>"
-    if mime_version is None:
-      mime_version = "1.0"
-    if plural_forms is None:
-      plural_forms = "nplurals=INTEGER; plural=EXPRESSION;"
-    if report_msgid_bugs_to is None:
-      report_msgid_bugs_to = ""
-    defaultargs = {
-      "Project-Id-Version":  project_id_version,
-      "Report-Msgid-Bugs-To":  report_msgid_bugs_to,
-      "POT-Creation-Date":  pot_creation_date,
-      "PO-Revision-Date":  po_revision_date,
-      "Last-Translator":  last_translator,
-      "Language-Team":  language_team,
-      "MIME-Version":  mime_version,
-      "Content-Type": "text/plain; charset=%s" % charset,
-      "Content-Transfer-Encoding":  encoding,
-      "Plural-Forms":  plural_forms,
-      "X-Generator": self.x_generator,
-      }
-    for key in self.header_order:
-      value = headerargs.pop(key, defaultargs[key])
-      headeritems.append("%s: %s\\n" % (key, value))
-    for key, value in headerargs.iteritems():
-      headeritems.append("%s: %s\\n" % (key, value))
-    headerpo.msgstr = [quote.quotestr(headerstr) for headerstr in headeritems]
+    headeritems = self.makeheaderdict(**kwargs)
+    headerpo.msgstr = ['""']
+    for (key, value) in headeritems.items():
+        headerpo.msgstr.append(quote.quotestr("%s: %s\\n" % (key, value)))
     return headerpo
 
-  def parseheader(self):
-    """parses the values in the header into a dictionary"""
-    headervalues = dictutils.ordereddict()
-    header = self.header()
-    if not header:
-      return headervalues
-    for entry in header.target.split("\n"):
-      if not entry or ":" not in entry: 
-        continue
-      key, value = entry.split(":", 1)
-      #We don't want unicode keys
-      key = str(key.strip())
-      headervalues[key] = value.strip()
-    return headervalues
-
-  def updateheader(self, add=False, **kwargs):
-    """update field(s) in the PO header"""
-    headeritems = self.parseheader()
-    if not headeritems and not add:
-      return
-    header = self.header()
-    if not header:
-      header = self.makeheader(**kwargs)
-      self.units.insert(0, header)
-      headeritems = self.parseheader()
-    else:
-      headerargs = dictutils.ordereddict()
-      fixedargs = dictutils.cidict()
-      for key, value in kwargs.items():
-        key = key.replace("_", "-")
-        if key.islower():
-          key = key.title()
-        fixedargs[key] = value
-      for key in self.header_order:
-        if key in fixedargs:
-          headerargs[key] = fixedargs.pop(key)
-      for key in fixedargs:
-        headerargs[key] = kwargs[key]
-      for key, value in headerargs.iteritems():
-        if headeritems.has_key(key) or add:
-          headeritems[key] = value
-    headerlines = [""]
-    for key, value in headeritems.items():
-      headerlines.append("%s: %s\\n" % (key, value))
-    header.msgstr = [quote.quotestr(headerline) for headerline in headerlines]
-    header.markfuzzy(False)
-    return header
-
-  def updateheaderplural(self, nplurals, plural):
-    """update the Plural-Form PO header"""
-    if isinstance(nplurals, basestring):
-      nplurals = int(nplurals)
-    self.updateheader( Plural_Forms = "nplurals=%d; plural=%s;" % (nplurals, plural) )
-
-  def getheaderplural(self):
-    """returns the nplural and plural values from the header"""
-    pluralformvalue = self.parseheader().get('Plural-Forms', None)
-    if pluralformvalue is None:
-      return None, None
-    nplural = sre.findall("nplurals=(.+?);", pluralformvalue)
-    plural = sre.findall("plural=(.+?);?$", pluralformvalue)
-    if not nplural or nplural[0] == "INTEGER":
-      nplural = None
-    else:
-      nplural = nplural[0]
-    if not plural or plural[0] == "EXPRESSION":
-      plural = None
-    else:
-      plural = plural[0]
-    return nplural, plural
-
   def changeencoding(self, newencoding):
     """changes the encoding on the file"""
     self.encoding = encodingToUse(newencoding)
@@ -833,7 +815,7 @@ class pofile(base.TranslationStore):
     if charsetline is None:
       headerstr += "Content-Type: text/plain; charset=%s" % self.encoding
     else:
-      charset = sre.search("charset=([^ ]*)", charsetline)
+      charset = re.search("charset=([^ ]*)", charsetline)
       if charset is None:
         newcharsetline = charsetline
         if not newcharsetline.strip().endswith(";"):
@@ -845,20 +827,6 @@ class pofile(base.TranslationStore):
       headerstr = headerstr.replace(charsetline, newcharsetline, 1)
     header.msgstr = quoteforpo(headerstr)
 
-  def isempty(self):
-    """returns whether the po file doesn't contain any definitions..."""
-    if len(self.units) == 0:
-      return 1
-    # first we check the header...
-    header = self.units[0]
-    if not (header.isheader() or header.isblank()):
-      return 0
-    # if there are any other units, this is only empty if they are blank
-    for thepo in self.units[1:]:
-      if not thepo.isblank():
-        return 0
-    return 1
-
   def parsestring(cls, storestring):
     """Parses the po file contents in the storestring and returns a new pofile object (classmethod, constructor)"""
     parsedfile = pofile()
@@ -884,34 +852,25 @@ class pofile(base.TranslationStore):
     linesprocessed = 0
     while end <= len(lines):
       if (end == len(lines)) or (not lines[end].strip()):   # end of lines or blank line
-        finished = 0
-        while not finished:
-          newpe = self.UnitClass(encoding=self.encoding)
-          linesprocessed = newpe.parse("\n".join(lines[start:end]))
-          start += linesprocessed
-          # TODO: find a better way of working out if we actually read anything
-          if linesprocessed >= 1 and newpe.getoutput():
-            self.units.append(newpe)
-            if newpe.isheader():
-              headervalues = self.parseheader()
-              contenttype = headervalues.get("Content-Type", None)
-              if contenttype is not None:
-                charsetmatch = sre.search("charset=([^ ]*)", contenttype)
-                if charsetmatch:
-                  self.encoding = charsetmatch.group(1)
-                # now that we know the encoding, decode the whole file
-                if self.encoding is not None and self.encoding.lower() != 'charset':
-                  lines = self.decode(lines)
-            if self.encoding is None: #still have not found an encoding, let's assume UTF-8
-              #TODO: This might be dead code
-              self.encoding = 'utf-8'
+        newpe = self.UnitClass(encoding=self.encoding)
+        linesprocessed = newpe.parse("\n".join(lines[start:end]))
+        start += linesprocessed
+        # TODO: find a better way of working out if we actually read anything
+        if linesprocessed >= 1 and newpe.getoutput():
+          self.units.append(newpe)
+          if newpe.isheader():
+            if "Content-Type" in self.parseheader():
+              self.encoding = newpe.encoding
+            # now that we know the encoding, decode the whole file
+            if self.encoding is not None and self.encoding.lower() != 'charset':
               lines = self.decode(lines)
-              self.units = []
-              start = 0
-              end = 0
-              finished = 1
-          else:
-            finished = 1
+          if self.encoding is None: #still have not found an encoding, let's assume UTF-8
+            #TODO: This might be dead code
+            self.encoding = 'utf-8'
+            lines = self.decode(lines)
+            self.units = []
+            start = 0
+            end = 0
       end = end+1
 
   def removeblanks(self):
@@ -954,6 +913,13 @@ class pofile(base.TranslationStore):
             addcomment(origpo)
           addcomment(thepo)
           uniqueelements.append(thepo)
+        elif duplicatestyle == "msgctxt":
+          origpo = msgiddict[msgid]
+          if origpo not in markedpos:
+            origpo.msgctxt.append('"%s"' % " ".join(origpo.getlocations()))
+            markedpos.append(thepo)
+          thepo.msgctxt.append('"%s"' % " ".join(thepo.getlocations()))
+          uniqueelements.append(thepo)
       else:
         if not msgid and duplicatestyle != "keep":
           addcomment(thepo)
@@ -1009,6 +975,11 @@ class pofile(base.TranslationStore):
     # TODO: make them unquoted strings, if useful...
     return dict([(" ".join(poel.msgid), poel) for poel in self.units])
 
+  def unit_iter(self):
+    for unit in self.units:
+      if not (unit.isheader() or unit.isobsolete()):
+        yield unit
+
 if __name__ == '__main__':
   import sys
   pf = pofile(sys.stdin)
