@@ -5,6 +5,7 @@ from translate.storage import po
 from translate.storage import test_base
 from translate.misc.multistring import multistring
 from translate.misc import wStringIO
+from py.test import raises
 
 def test_roundtrip_quoting():
     specials = ['Fish & chips', 'five < six', 'six > five', 
@@ -73,7 +74,10 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.addnote("Thank you", origin="translator")
         assert str(unit) == '# Which meaning of file?\n# Thank you\n#. Verb\nmsgid "File"\nmsgstr ""\n'
 
+        assert unit.getnotes("developer") == "Verb"
+        assert unit.getnotes("translator") == "Which meaning of file?\nThank you"
         assert unit.getnotes() == "Which meaning of file?\nThank you\nVerb"
+        assert raises(ValueError, unit.getnotes, "devteam")
 
     def test_notes_withcomments(self):
         """tests that when we add notes that look like comments that we treat them properly"""
@@ -81,7 +85,116 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.addnote("# Double commented comment")
         assert str(unit) == '# # Double commented comment\nmsgid "File"\nmsgstr ""\n'
         assert unit.getnotes() == "# Double commented comment"
+
+    def test_adding_empty_note(self):
+        unit = self.UnitClass("bla")
+        assert not '#' in str(unit)
+        unit.addnote("")
+        assert not '#' in str(unit)
         
+    def test_no_plural_settarget(self):
+	"""tests that target handling of file with no plural is correct"""
+	# plain text, no plural test
+	unit = self.UnitClass("Tree")
+	unit.target = "ki"
+	assert unit.target.strings == ["ki"]
+	assert unit.source.strings == ["Tree"]
+	assert unit.hasplural() == False
+
+	# plural test with multistring
+	unit.setsource(["Tree", "Trees"])
+	assert unit.source.strings == ["Tree", "Trees"]
+	assert unit.hasplural()
+	unit.target = multistring(["ki", "ni ki"])
+	assert unit.target.strings == ["ki", "ni ki"]
+
+	# test of msgid with no plural and msgstr with plural
+	unit = self.UnitClass("Tree")
+	assert raises(ValueError, unit.settarget, [u"ki", u"ni ki"])
+	assert unit.hasplural() == False
+
+    def test_wrap_firstlines(self):
+        '''tests that we wrap the first line correctly a first line if longer then 71 chars
+        as at 71 chars we should align the text on the left and preceed with with a msgid ""'''
+        # longest before we wrap text
+        str_max = "123456789 123456789 123456789 123456789 123456789 123456789 123456789 1"
+        unit = self.UnitClass(str_max)
+        expected = 'msgid "%s"\nmsgstr ""\n' % str_max
+        print expected, str(unit)
+        assert str(unit) == expected
+        # at this length we wrap
+        str_wrap = str_max + '2'
+        unit = self.UnitClass(str_wrap)
+        expected = 'msgid ""\n"%s"\nmsgstr ""\n' % str_wrap
+        print expected, str(unit)
+        assert str(unit) == expected
+
+    def test_wrap_on_newlines(self):
+        """test that we wrap newlines on a real \n"""
+        string = "123456789\n" * 3
+        postring = ('"123456789\\n"\n' * 3)[:-1]
+        unit = self.UnitClass(string)
+        expected = 'msgid ""\n%s\nmsgstr ""\n' % postring
+        print expected, str(unit)
+        assert str(unit) == expected
+
+        # Now check for long newlines segments
+        longstring = ("123456789 " * 10 + "\n") * 3
+        expected = r'''msgid ""
+"123456789 123456789 123456789 123456789 123456789 123456789 123456789 "
+"123456789 123456789 123456789 \n"
+"123456789 123456789 123456789 123456789 123456789 123456789 123456789 "
+"123456789 123456789 123456789 \n"
+"123456789 123456789 123456789 123456789 123456789 123456789 123456789 "
+"123456789 123456789 123456789 \n"
+msgstr ""
+'''
+        unit = self.UnitClass(longstring)
+        print expected, str(unit)
+        assert str(unit) == expected
+
+    def test_wrap_on_max_line_length(self):
+        """test that we wrap all lines on the maximum line length"""
+        string = "1 3 5 7 N " * 11
+        expected = 'msgid ""\n%s\nmsgstr ""\n' % '"1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 "\n"7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N "'
+        unit = self.UnitClass(string)
+        print "Expected:"
+        print expected
+        print "Actual:"
+        print str(unit)
+        assert str(unit) == expected
+
+    def test_spacing_max_line(self):
+        """Test that the spacing of text is done the same as msgcat."""
+        idstring = "Creates a new document using an existing template iiiiiiiiiiiiiiiiiiiiiii or "
+        idstring += "opens a sample document."
+        expected = '''msgid ""
+"Creates a new document using an existing template iiiiiiiiiiiiiiiiiiiiiii or "
+"opens a sample document."
+msgstr ""
+'''
+        unit = self.UnitClass(idstring)
+        print "Expected:"
+        print expected
+        print "Actual:"
+        print str(unit)
+        assert str(unit) == expected
+
+    def test_wrapping_bug(self):
+        """This tests for a wrapping bug that existed at some stage."""
+        unit = self.UnitClass("")
+        message = 'Projeke ya Pootle ka boyona e ho <a href="http://translate.sourceforge.net/">translate.sourceforge.net</a> moo o ka fumanang dintlha ka source code, di mailing list jwalo jwalo.'
+        unit.target = message
+        print unit.target
+        assert unit.target == message
+
+    def test_extract_msgidcomments_from_text(self):
+        """Test that KDE style comments are extracted correctly."""
+        unit = self.UnitClass("test source")
+
+        kdetext = "_: Simple comment\nsimple text"
+        assert unit.extract_msgidcomments(kdetext) == "Simple comment"
+
 class TestPO(test_base.TestTranslationStore):
     StoreClass = po.pofile
     def poparse(self, posource):
@@ -94,7 +207,7 @@ class TestPO(test_base.TestTranslationStore):
         """helper that converts po source to pofile object and back"""
         return str(self.poparse(posource))
 
-    def pomerge(self, oldmessage, newmessage):
+    def pomerge(self, oldmessage, newmessage, authoritative):
         """helper that merges two messages"""
         dummyfile = wStringIO.StringIO(oldmessage)
         oldpofile = po.pofile(dummyfile)
@@ -105,7 +218,7 @@ class TestPO(test_base.TestTranslationStore):
           newunit = newpofile.units[0]
         else:
           newunit = po.pounit()
-        oldunit.merge(newunit)
+        oldunit.merge(newunit, authoritative=authoritative)
         print oldunit
         return str(oldunit)
 
@@ -119,6 +232,20 @@ class TestPO(test_base.TestTranslationStore):
         assert thepo.source == "test"
         assert thepo.target == "rest"
 
+    def test_copy(self):
+        """checks that we can copy all the needed PO fields"""
+        posource = '''# TRANSLATOR-COMMENTS
+#. AUTOMATIC-COMMENTS
+#: REFERENCE...
+#, fuzzy
+msgctxt "CONTEXT"
+msgid "UNTRANSLATED-STRING"
+msgstr "TRANSLATED-STRING"'''
+        pofile = self.poparse(posource)
+        oldunit = pofile.units[0]
+        newunit = oldunit.copy()
+        assert newunit == oldunit
+
     def test_combine_msgidcomments(self):
         """checks that we don't get duplicate msgid comments"""
         posource = 'msgid "test me"\nmsgstr ""'
@@ -138,6 +265,105 @@ class TestPO(test_base.TestTranslationStore):
         assert len(pofile.units) == 1
         assert pofile.units[0].getlocations() == ["source1", "source2"]
 
+    def test_merge_duplicates_msgctxt(self):
+        """checks that merging duplicates works for msgctxt"""
+        posource = '#: source1\nmsgid "test me"\nmsgstr ""\n\n#: source2\nmsgid "test me"\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        pofile.removeduplicates("msgctxt")
+        print pofile
+        assert len(pofile.units) == 2
+        assert str(pofile.units[0]).count("source1") == 2
+        assert str(pofile.units[1]).count("source2") == 2
+  
+    def test_parse_context(self):
+        """Tests that msgctxt is parsed correctly and that it is accessible via the api methods."""
+        posource = '''# Test comment
+#: source1
+msgctxt "noun"
+msgid "convert"
+msgstr "bekeerling"
+
+# Test comment 2
+#: source2
+msgctxt "verb"
+msgid "convert"
+msgstr "omskakel"
+'''
+        pofile = self.poparse(posource)
+        unit = pofile.units[0]
+
+        assert unit.getcontext() == 'noun'
+        assert unit.getnotes() == 'Test comment'
+
+        unit = pofile.units[1]
+        assert unit.getcontext() == 'verb'
+        assert unit.getnotes() == 'Test comment 2'
+
+
+    def test_parse_advanced_context(self):
+        """Tests that some weird possible msgctxt scenarios are parsed correctly."""
+        posource = r'''# Test multiline context
+#: source1
+msgctxt "Noun."
+" A person that changes his or her ways."
+msgid "convert"
+msgstr "bekeerling"
+
+# Test quotes
+#: source2
+msgctxt "Verb. Converting from \"something\" to \"something else\"."
+msgid "convert"
+msgstr "omskakel"
+
+# Test quotes, newlines and multiline.
+#: source3
+msgctxt "Verb.\nConverting from \"something\""
+" to \"something else\"."
+msgid "convert"
+msgstr "omskakel"
+'''
+        pofile = self.poparse(posource)
+        unit = pofile.units[0]
+
+        assert unit.getcontext() == 'Noun. A person that changes his or her ways.'
+        assert unit.getnotes() == 'Test multiline context'
+
+        unit = pofile.units[1]
+        assert unit.getcontext() == 'Verb. Converting from "something" to "something else".'
+        assert unit.getnotes() == 'Test quotes'
+        
+        unit = pofile.units[2]
+        assert unit.getcontext() == 'Verb.\nConverting from "something" to "something else".'
+        assert unit.getnotes() == 'Test quotes, newlines and multiline.'
+ 
+    def test_kde_context(self):
+        """Tests that kde-style msgid comments can be retrieved via getcontext()."""
+        posource = '''# Test comment
+#: source1
+msgid ""
+"_: Noun\n"
+"convert"
+msgstr "bekeerling"
+
+# Test comment 2
+#: source2
+msgid ""
+"_: Verb. _: "
+"The action of changing.\n"
+"convert"
+msgstr "omskakel"
+'''
+        pofile = self.poparse(posource)
+        unit = pofile.units[0]
+
+        assert unit.getcontext() == 'Noun'
+        assert unit.getnotes() == 'Test comment'
+
+        unit = pofile.units[1]
+        assert unit.getcontext() == 'Verb. _: The action of changing.'
+        assert unit.getnotes() == 'Test comment 2'
+
     def test_merge_mixed_sources(self):
         """checks that merging works with different source location styles"""
         posource = '''
@@ -259,6 +485,40 @@ msgstr[0] "Sheep"
         assert unit.target == "Sheep"
         assert unit.target.strings == ["Sheep"]
 
+    def test_plural_unicode(self):
+        """tests that all parts of the multistring are unicode."""
+        posource = r'''msgid "Ców"
+msgid_plural "Cóws"
+msgstr[0] "Kóei"
+msgstr[1] "Kóeie"
+'''
+        pofile = po.pofile(wStringIO.StringIO(posource))
+        unit = pofile.units[0]
+        assert isinstance(unit.source, multistring)
+        assert isinstance(unit.source.strings[1], unicode)
+        
+
+    def wtest_kde_plurals(self):
+        """Tests kde-style plurals. (Bug: 191)"""
+        posource = '''msgid "_n Singular\n"
+"Plural"
+msgstr "Een\n"
+"Twee\n"
+"Drie"
+'''
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 1
+        unit = pofile.units[0]
+	assert unit.hasplural() == True
+        assert isinstance(unit.source, multistring)
+        print unit.source.strings
+        assert unit.source == "Singular"
+        assert unit.source.strings == ["Singular", "Plural"]
+        assert isinstance(unit.target, multistring)
+        print unit.target.strings
+        assert unit.target == "Een"
+        assert unit.target.strings == ["Een", "Twee", "Drie"]
+
     def test_posections(self):
         """checks the content of all the expected sections of a PO message"""
         posource = '# other comment\n#. automatic comment\n#: source comment\n#, fuzzy\nmsgid "One"\nmsgstr "Een"\n'
@@ -271,6 +531,19 @@ msgstr[0] "Sheep"
         assert pofile.units[0].sourcecomments == ["#: source comment\n"]
         assert pofile.units[0].typecomments == ["#, fuzzy\n"]
 
+    def test_empty_lines_notes(self):
+        """Tests that empty comment lines are preserved"""
+        posource = r'''# License name
+#
+# license line 1
+# license line 2
+# license line 3
+msgid ""
+msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
+'''
+        pofile = self.poparse(posource)
+        assert str(pofile) == posource
+
     def test_fuzzy(self):
         """checks that fuzzy functionality works as expected"""
         posource = '#, fuzzy\nmsgid "ball"\nmsgstr "bal"\n'
@@ -314,6 +587,8 @@ msgstr[0] "Sheep"
         print pofile
         assert str(unit) == poexpected
         
+    def test_makeobsolete_plural(self):
+        """Tests making a plural unit obsolete"""
         posource = r'''msgid "Cow"
 msgid_plural "Cows"
 msgstr[0] "Koei"
@@ -333,6 +608,19 @@ msgstr[1] "Koeie"
         print pofile
         assert str(unit) == poexpected
 
+    def test_makeobsolete_msgctxt(self):
+        """Tests making a unit with msgctxt obsolete"""
+        posource = '#: test.c\nmsgctxt "Context"\nmsgid "test"\nmsgstr "rest"\n'
+        poexpected = '#~ msgctxt "Context"\n#~ msgid "test"\n#~ msgstr "rest"\n'
+        pofile = self.poparse(posource)
+        print pofile
+        unit = pofile.units[0]
+        assert not unit.isobsolete()
+        unit.makeobsolete()
+        assert unit.isobsolete()
+        print pofile
+        assert str(unit) == poexpected
+
     def test_makeobsolete_msgidcomments(self):
         """Tests making a unit with msgidcomments obsolete"""
         posource = '#: first.c\nmsgid ""\n"_: first.c\\n"\n"test"\nmsgstr "rest"\n\n#: second.c\nmsgid ""\n"_: second.c\\n"\n"test"\nmsgstr "rest"'
@@ -374,7 +662,7 @@ msgstr[1] "Koeie"
         oldsource = '#. old comment\n#: line:10\nmsgid "One"\nmsgstr "Een"\n'
         newsource = '#. new comment\n#: line:10\nmsgid "One"\nmsgstr ""\n'
         expected = '#. new comment\n#: line:10\nmsgid "One"\nmsgstr "Een"\n'
-        assert self.pomerge(newsource, oldsource) == expected
+        assert self.pomerge(newsource, oldsource, authoritative=True) == expected
 
     def test_unassociated_comments(self):
         """tests behaviour of unassociated comments."""
@@ -383,78 +671,3 @@ msgstr[1] "Koeie"
         print "__str__", str(oldfile)
         assert len(oldfile.units) == 2
         assert str(oldfile).find("# old lonesome comment\n\n") >= 0
-
-    def test_header_blank(self):
-        """test header functionality"""
-        posource = r'''# other comment\n
-msgid ""
-msgstr ""
-"Project-Id-Version: PACKAGE VERSION\n"
-"Report-Msgid-Bugs-To: \n"
-"POT-Creation-Date: 2006-03-08 17:30+0200\n"
-"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
-"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
-"Language-Team: LANGUAGE <LL@li.org>\n"
-"MIME-Version: 1.0\n"
-"Content-Type: text/plain; charset=UTF-8\n"
-"Content-Transfer-Encoding: 8bit\n"
-"Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;\n"
-'''
-        pofile = self.poparse(posource)
-        print pofile
-        assert len(pofile.units) == 1
-        header = pofile.units[0]
-        assert header.isheader()
-        assert not header.isblank()
-
-        headeritems = pofile.parseheader()
-        assert headeritems["Project-Id-Version"] == "PACKAGE VERSION"
-        assert headeritems["Report-Msgid-Bugs-To"] == ""
-        #assert headeritems["POT-Creation-Date"] == ""
-        assert headeritems["PO-Revision-Date"] == "YEAR-MO-DA HO:MI+ZONE"
-        assert headeritems["Last-Translator"] == "FULL NAME <EMAIL@ADDRESS>"
-        assert headeritems["Language-Team"] == "LANGUAGE <LL@li.org>"
-        assert headeritems["MIME-Version"] == "1.0"
-        assert headeritems["Content-Type"] == "text/plain; charset=UTF-8"
-        assert headeritems["Content-Transfer-Encoding"] == "8bit"
-        assert headeritems["Plural-Forms"] == "nplurals=INTEGER; plural=EXPRESSION;"
-        
-    def test_plural_equation(self):
-        """test that we work with the equation even is the last semicolon is left out, since gettext
-        tools don't seem to mind"""
-        posource = r'''msgid ""
-msgstr ""
-"Plural-Forms: nplurals=2; plural=(n != 1)%s\n"
-'''
-        for colon in ("", ";"):
-          pofile = self.poparse(posource % colon)
-          print pofile
-          assert len(pofile.units) == 1
-          header = pofile.units[0]
-          assert header.isheader()
-          assert not header.isblank()
-
-          headeritems = pofile.parseheader()
-          nplural, plural = pofile.getheaderplural()
-          assert nplural == "2"
-          assert plural == "(n != 1)"
-
-    def test_plural_equation_across_lines(self):
-        """test that we work if the plural equation spans more than one line"""
-        posource = r'''msgid ""
-msgstr ""
-"Plural-Forms:  nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%"
-"10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);\n"
-'''
-        pofile = self.poparse(posource)
-        print pofile
-        assert len(pofile.units) == 1
-        header = pofile.units[0]
-        assert header.isheader()
-        assert not header.isblank()
-
-        headeritems = pofile.parseheader()
-        nplural, plural = pofile.getheaderplural()
-        assert nplural == "3"
-        assert plural == "(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2)"
-
