@@ -20,58 +20,19 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-"""module for parsing .xliff files for translation"""
+"""Module for handling XLIFF files for translation.
 
+The official recommendation is to use the extention .xlf for XLIFF files.
+"""
+
+from translate.storage import base
 from translate.storage import lisa
-from xml.dom import minidom
-
-def writexml(self, writer, indent="", addindent="", newl=""):
-    """a replacement to writexml that formats it more like typical .ts files"""
-    # indent = current indentation
-    # addindent = indentation to add to higher levels
-    # newl = newline string
-    writer.write(indent+"<" + self.tagName)
-
-    attrs = self._get_attributes()
-    a_names = attrs.keys()
-    a_names.sort()
-
-    for a_name in a_names:
-        writer.write(" %s=\"" % a_name)
-        minidom._write_data(writer, attrs[a_name].value)
-        writer.write("\"")
-    if self.childNodes:
-        if len(self.childNodes) == 1 and self.childNodes[0].nodeType == self.TEXT_NODE:
-          writer.write(">")
-          for node in self.childNodes:
-              node.writexml(writer,"","","")
-          writer.write("</%s>%s" % (self.tagName,newl))
-        else:
-          writer.write(">%s"%(newl))
-          for node in self.childNodes:
-              node.writexml(writer,indent+addindent,addindent,newl)
-          writer.write("%s</%s>%s" % (indent,self.tagName,newl))
-    else:
-        writer.write("/>%s"%(newl))
-
-# commented out modifications to minidom classes
-'''
-Element_writexml = minidom.Element.writexml
-for elementclassname in dir(minidom):
-  elementclass = getattr(minidom, elementclassname)
-  if not isinstance(elementclass, type(minidom.Element)):
-    continue
-  if not issubclass(elementclass, minidom.Element):
-    continue
-  if elementclass.writexml != Element_writexml:
-    continue
-  elementclass.writexml = writexml
-'''
 
 # TODO: handle translation types
 
 class xliffunit(lisa.LISAunit):
-    """A single term in the xliff file.""" 
+    """A single term in the xliff file."""
+
     rootNode = "trans-unit"
     languageNode = "source"
     textNode = ""
@@ -79,7 +40,8 @@ class xliffunit(lisa.LISAunit):
     #TODO: id and all the trans-unit level stuff
 
     def createlanguageNode(self, lang, text, purpose):
-        """returns an xml Element setup with given parameters"""
+        """Returns an xml Element setup with given parameters."""
+
         #TODO: for now we do source, but we have to test if it is target, perhaps 
         # with parameter. Alternatively, we can use lang, if supplied, since an xliff 
         #file has to conform to the bilingual nature promised by the header.
@@ -95,6 +57,7 @@ class xliffunit(lisa.LISAunit):
     
     def getlanguageNodes(self):
         """We override this to get source and target nodes."""
+
         sources = self.xmlelement.getElementsByTagName(self.languageNode)
         targets = self.xmlelement.getElementsByTagName("target")
         sourcesl = len(sources)
@@ -106,54 +69,163 @@ class xliffunit(lisa.LISAunit):
             nodes.extend(sources[- (sourcesl - targetsl):])
         return nodes
 
+    def addalttrans(self, txt, origin=None, lang=None):
+        """Adds an alt-trans tag and alt-trans components to the unit.
+        
+        @type txt: String
+        @param txt: Alternative translation of the source text.
+        """
+
+        #TODO: support adding a source tag ad match quality attribute.  At 
+        # the source tag is needed to inject fuzzy matches from a TM.
+        if isinstance(txt, str):
+            txt = txt.decode("utf-8")
+        alttarget = self.document.createElement("target")
+        alttarget.appendChild(self.document.createTextNode(txt))
+        alttrans = self.document.createElement("alt-trans")
+        alttrans.appendChild(alttarget)
+        if origin:
+            alttrans.setAttribute("origin", origin)
+        if lang:
+            alttrans.setAttribute("xml:lang", lang)
+        self.xmlelement.appendChild(alttrans)
+
+    def getalttrans(self, origin=None):
+        """Returns <alt-trans> for the given origin as a list of units. No 
+        origin means all alternatives."""
+        nodes = self.xmlelement.getElementsByTagName("alt-trans")
+        translist = []
+        for i in range(len(nodes)):
+            if self.correctorigin(nodes[i], origin):
+                # We build some mini units that keep the xmlelement. This 
+                # makes it easier to delete it if it is passed back to us.
+                newunit = base.TranslationUnit(self.source)
+                node = nodes[i]
+
+                # the source tag is optional
+                sourcelist = node.getElementsByTagName("source")
+                if sourcelist:
+                    newunit.source = lisa.getText(sourcelist[0])
+
+                # must have one or more targets
+                targetlist = node.getElementsByTagName("target")
+                newunit.target = lisa.getText(targetlist[0])
+                #TODO: support multiple targets better
+                #TODO: support notes in alt-trans
+                newunit.xmlelement = node
+
+                translist.append(newunit)
+        return translist
+
+    def delalttrans(self, alternative):
+        """Removes the supplied alternative from the list of alt-trans tags"""
+        self.xmlelement.removeChild(alternative.xmlelement)
+
     def addnote(self, text, origin=None):
         """Add a note specifically in a "note" tag"""
         if isinstance(text, str):
             text = text.decode("utf-8")
         note = self.document.createElement("note")
-        note.appendChild(self.document.createTextNode(text))
+        note.appendChild(self.document.createTextNode(text.strip()))
         if origin:
             note.setAttribute("from", origin)
         self.xmlelement.appendChild(note)        
 
-    def getnotes(self):
-        """Returns the text from all the notes"""
-        return lisa.getText(self.xmlelement.getElementsByTagName("note"))
+    def getnotelist(self, origin=None):
+        """Private method that returns the text from notes matching 'origin' or all notes."""
+        notenodes = self.xmlelement.getElementsByTagName("note")
+        initial_list = [lisa.getText(note) for note in notenodes if self.correctorigin(note, origin)]
+
+        # Remove duplicate entries from list:
+        dictset = {}
+        notelist = [dictset.setdefault(note, note) for note in initial_list if note not in dictset]
+
+        return notelist 
+
+    def getnotes(self, origin=None):
+        return '\n'.join(self.getnotelist(origin=origin)) 
+
+    def removenotes(self):
+        """Remove all the translator notes."""
+        notes = self.xmlelement.getElementsByTagName("note")
+        for note in notes:
+            if self.correctorigin(note, origin="translator"):
+                self.xmlelement.removeChild(note)
+
+    def adderror(self, errorname, errortext):
+        """Adds an error message to this unit."""
+        text = errorname + ': ' + errortext
+        self.addnote(text, origin="pofilter")
+
+    def geterrors(self):
+        """Get all error messages."""
+        notelist = self.getnotelist(origin="pofilter")
+        errordict = {}
+        for note in notelist:
+            errorname, errortext = note.split(': ')
+            errordict[errorname] = errortext
+        return errordict
 
     def isapproved(self):
+        """States whether this unit is approved"""
         return self.xmlelement.getAttribute("approved") == "yes"
 
-    def isfuzzy(self):
+    def isreview(self):
+        """States whether this unit needs to be reviewed"""
         targetnode = self.getlanguageNode(lang=None, index=1)
         return not targetnode is None and \
-                (targetnode.getAttribute("state-qualifier") == "fuzzy-match" or \
-                targetnode.getAttribute("state") == "needs-review-translation")
+                "needs-review" in targetnode.getAttribute("state")
+
+    def markreviewneeded(self, needsreview=True, explanation=None):
+        """Marks the unit to indicate whether it needs review. Adds an optional explanation as a note."""
+        targetnode = self.getlanguageNode(lang=None, index=1)
+        if targetnode:
+            if needsreview:
+                targetnode.setAttribute("state", "needs-review-translation")
+                if explanation:
+                    self.addnote(explanation, origin="translator")
+            else:
+                targetnode.removeAttribute("state")
+
+    def isfuzzy(self):
+#        targetnode = self.getlanguageNode(lang=None, index=1)
+#        return not targetnode is None and \
+#                (targetnode.getAttribute("state-qualifier") == "fuzzy-match" or \
+#                targetnode.getAttribute("state") == "needs-review-translation")
+        return not self.isapproved()
                 
     def markfuzzy(self, value=True):
+        if value:
+            self.xmlelement.setAttribute("approved", "no")
+        else:
+            self.xmlelement.setAttribute("approved", "yes")
         targetnode = self.getlanguageNode(lang=None, index=1)
         if targetnode:
             if value:
                 targetnode.setAttribute("state", "needs-review-translation")
-                targetnode.setAttribute("state-qualifier", "fuzzy-match")
-            elif self.isfuzzy():                
-                targetnode.removeAttribute("state")
-                targetnode.removeAttribute("state-qualifier")
-        elif value:
-            #If there is no target, we can't really indicate fuzzyness, so we set
-            #approved to "no", but we don't take it into account in isfuzzy()
-            #TODO: review decision
-            self.xmlelement.setAttribute("approved", "no")
+            else:
+                for attribute in ["state", "state-qualifier"]:
+                    if targetnode.hasAttribute(attribute):
+                        targetnode.removeAttribute(attribute)
 
-    def istranslated(self):
-        targetnode = self.getlanguageNode(lang=None, index=1)
-        return not targetnode is None and \
-                (targetnode.getAttribute("state") == "translated")
+    def settarget(self, text, lang='xx', append=False):
+        """Sets the target string to the given value."""
+        super(xliffunit, self).settarget(text, lang, append)
+        if text:
+            self.marktranslated()
+
+# This code is commented while this will almost always return false.
+# This way pocount, etc. works well.
+#    def istranslated(self):
+#        targetnode = self.getlanguageNode(lang=None, index=1)
+#        return not targetnode is None and \
+#                (targetnode.getAttribute("state") == "translated")
 
     def marktranslated(self):
         targetnode = self.getlanguageNode(lang=None, index=1)
         if not targetnode:
             return
-        if self.isfuzzy():
+        if self.isfuzzy() and targetnode.hasAttribute("state-qualifier"):
             #TODO: consider
             targetnode.removeAttribute("state-qualifier")
         targetnode.setAttribute("state", "translated")
@@ -164,6 +236,9 @@ class xliffunit(lisa.LISAunit):
     def getid(self):
         return self.xmlelement.getAttribute("id")
 
+    def getlocations(self):
+        return [self.getid()]
+
     def createcontextgroup(self, name, contexts=None, purpose=None):
         """Add the context group to the trans-unit with contexts a list with
         (type, text) tuples describing each context."""
@@ -207,6 +282,16 @@ class xliffunit(lisa.LISAunit):
         if otherunit.isfuzzy():
             self.markfuzzy()
 
+    def correctorigin(self, node, origin):
+        """Check against node tag's origin (e.g note or alt-trans)"""
+        if origin == None:
+            return True
+        elif origin in node.getAttribute("from"):
+            return True
+        elif origin in node.getAttribute("origin"):
+            return True
+        else:
+            return False
 
 class xlifffile(lisa.LISAfile):
     """Class representing a XLIFF file store."""
@@ -226,19 +311,34 @@ class xlifffile(lisa.LISAfile):
         self._filename = "NoName"
         self._messagenum = 0
 
+        # Allow the inputfile to override defaults for source and target language.
+        filenode = self.document.getElementsByTagName('file')[0]
+        sourcelanguage = filenode.getAttribute('source-language')
+        if sourcelanguage:
+            self.setsourcelanguage(sourcelanguage)
+        targetlanguage = filenode.getAttribute('target-language')
+        if targetlanguage:
+            self.settargetlanguage(targetlanguage)
+
     def addheader(self):
         """Initialise the file header."""
         self.document.getElementsByTagName("file")[0].setAttribute("source-language", self.sourcelanguage)
+        if self.targetlanguage:
+            self.document.getElementsByTagName("file")[0].setAttribute("target-language", self.targetlanguage)
 
-    def createfilenode(self, filename, sourcelanguage=None, datatype='plaintext'):
+    def createfilenode(self, filename, sourcelanguage=None, targetlanguage=None, datatype='plaintext'):
         """creates a filenode with the given filename. All parameters are needed
         for XLIFF compliance."""
         self.removedefaultfile()
         if sourcelanguage is None:
             sourcelanguage = self.sourcelanguage
+        if targetlanguage is None:
+            targetlanguage = self.targetlanguage
         filenode = self.document.createElement("file")
         filenode.setAttribute("original", filename)
         filenode.setAttribute("source-language", sourcelanguage)
+        if targetlanguage:
+            filenode.setAttribute("target-language", targetlanguage)
         filenode.setAttribute("datatype", datatype)
         bodyNode = self.document.createElement(self.bodyNode)
         filenode.appendChild(bodyNode)
@@ -251,7 +351,10 @@ class xlifffile(lisa.LISAfile):
     def getfilenames(self):
         """returns all filenames in this XLIFF file"""
         filenodes = self.document.getElementsByTagName("file")
-        return [self.getfilename(filenode) for filenode in filenodes]
+        filenames = [self.getfilename(filenode) for filenode in filenodes]
+        if len(filenames) == 1 and filenames[0] == '':
+            filenames = []
+        return filenames
 
     def getfilenode(self, filename):
         """finds the filenode with the given name"""
@@ -287,13 +390,13 @@ class xlifffile(lisa.LISAfile):
 
     def getheadernode(self, filenode, createifmissing=False):
         """finds the header node for the given filenode"""
-        #Deprecated?
+        # TODO: Deprecated?
         headernodes = list(filenode.getElementsByTagName("header"))
         if headernodes:
             return headernodes[0]
         if not createifmissing:
             return None
-        headernode = minidom.Element("header")
+        headernode = ourdom.Element("header")
         filenode.appendChild(headernode)
         return headernode
 
@@ -355,20 +458,15 @@ class xlifffile(lisa.LISAfile):
         self.removedefaultfile()
         return super(xlifffile, self).__str__()
 
-    def parsefile(cls, storefile):
-        """Normal parsing, but if it smells like a PO-XLIFF, rather hand over to poxliff."""
-        if isinstance(storefile, basestring):
-            storefile = open(storefile, "r")
-        storestring = storefile.read()
-        xliff = cls.parsestring(storestring)
-
+    def parsestring(cls, storestring):
+        """Parses the string to return the correct file object"""
+        xliff = super(xlifffile, cls).parsestring(storestring)
         if xliff.units:
             header = xliff.units[0]
             if ("gettext-domain-header" in header.getrestype() or xliff.getdatatype() == "po") \
                     and cls.__name__.lower() != "poxlifffile":
                 import poxliff
-                storefile.seek(0)
-                xliff = poxliff.PoXliffFile.parsefile(storefile)
+                xliff = poxliff.PoXliffFile.parsestring(storestring)
         return xliff
-    parsefile = classmethod(parsefile)
+    parsestring = classmethod(parsestring)
 
