@@ -26,7 +26,6 @@ from translate.misc import optrecurse
 import optparse
 import sys
 import os
-import sre
 
 class ConflictOptionParser(optrecurse.RecursiveOptionParser):
   """a specialized Option Parser for the conflict tool..."""
@@ -185,5 +184,9 @@ def main():
   parser.add_option("", "--accelerator", dest="accelchars", default="",
     metavar="ACCELERATORS", help="ignores the given accelerator characters when matching")
   parser.set_usage()
+  parser.description=__doc__
   parser.run()
 
+
+if __name__ == '__main__':
+    main()
