@@ -19,46 +19,49 @@
 # along with translate; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""grep for gettext .po localization files"""
+"""grep for localization files in various formats, eg. gettext .po and xliff"""
 
-from translate.storage import po
+from translate.storage import factory
 from translate.misc import optrecurse
 from translate.misc.multistring import multistring
-import sre
+import re
 import locale
 
-class pogrepfilter:
-  def __init__(self, searchstring, searchparts, ignorecase=False, useregexp=False, invertmatch=False, accelchar=None, encoding='utf-8'):
-    """builds a pocheckfilter using the given checker"""
+class GrepFilter:
+  def __init__(self, searchstring, searchparts, ignorecase=False, useregexp=False, invertmatch=False, accelchar=None, encoding='utf-8', includeheader=False):
+    """builds a checkfilter using the given checker"""
     if isinstance(searchstring, unicode):
       self.searchstring = searchstring
     else:
       self.searchstring = searchstring.decode(encoding)
     if searchparts:
-      self.searchmsgid = "msgid" in searchparts
-      self.searchmsgstr = "msgstr" in searchparts
-      self.searchcomment = 'comment' in searchparts
-      self.searchsource = 'source' in searchparts
+      # For now we still support the old terminology, except for the old 'source'
+      # which has a new meaning now.
+      self.search_source = ('source' in searchparts) or ('msgid' in searchparts)
+      self.search_target = ('target' in searchparts) or ('msgstr' in searchparts)
+      self.search_notes =  ('notes' in searchparts) or ('comment' in searchparts)
+      self.search_locations = 'locations' in searchparts
     else:
-      self.searchmsgid = True
-      self.searchmsgstr = True
-      self.searchcomment = False
-      self.searchsource = False
+      self.search_source = True
+      self.search_target = True
+      self.search_notes = False
+      self.search_locations = False
     self.ignorecase = ignorecase
     if self.ignorecase:
       self.searchstring = self.searchstring.lower()
     self.useregexp = useregexp
     if self.useregexp:
-      self.searchpattern = sre.compile(self.searchstring)
+      self.searchpattern = re.compile(self.searchstring)
     self.invertmatch = invertmatch
     self.accelchar = accelchar
+    self.includeheader = includeheader
 
   def matches(self, teststr):
     if self.ignorecase:
       teststr = teststr.lower()
     if self.accelchar:
-      teststr = sre.sub(self.accelchar + self.accelchar, "#", teststr)
-      teststr = sre.sub(self.accelchar, "", teststr)
+      teststr = re.sub(self.accelchar + self.accelchar, "#", teststr)
+      teststr = re.sub(self.accelchar, "", teststr)
     if self.useregexp:
       found = self.searchpattern.search(teststr)
     else:
@@ -67,43 +70,47 @@ class pogrepfilter:
       found = not found
     return found
 
-  def filterelement(self, thepo):
+  def filterelement(self, unit):
     """runs filters on an element"""
-    if thepo.isheader(): return []
+    if unit.isheader(): return []
 
-    if self.searchmsgid:
-      if isinstance(thepo.source, multistring):
-        strings = thepo.source.strings
+    if self.search_source:
+      if isinstance(unit.source, multistring):
+        strings = unit.source.strings
       else:
-        strings = [thepo.source]
+        strings = [unit.source]
       for string in strings:
         if self.matches(string):
           return True
 
-    if self.searchmsgstr:
-      if isinstance(thepo.target, multistring):
-        strings = thepo.target.strings
+    if self.search_target:
+      if isinstance(unit.target, multistring):
+        strings = unit.target.strings
       else:
-        strings = [thepo.target]
+        strings = [unit.target]
       for string in strings:
         if self.matches(string):
           return True
 
-    if self.searchcomment:
-      if self.matches(" ".join(thepo.othercomments)): return True
-    if self.searchsource:
-      if self.matches(" ".join(thepo.sourcecomments)): return True
-          
+    if self.search_notes:
+      return self.matches(unit.getnotes(origin="translator"))
+    if self.search_locations:
+      return self.matches(" ".join(unit.getlocations()))
     return False
 
-  def filterfile(self, thepofile):
-    """runs filters on a file"""
-    thenewpofile = po.pofile()
-    for thepo in thepofile.units:
-      matches = self.filterelement(thepo)
-      if matches:
-        thenewpofile.units.append(thepo)
-    return thenewpofile
+  def filterfile(self, thefile):
+    """runs filters on a translation file object"""
+    thenewfile = type(thefile)()
+    for unit in thefile.units:
+      # filterelement() returns True when a unit matches.
+      if self.filterelement(unit):
+        thenewfile.addunit(unit)
+    if self.includeheader and thenewfile.units > 0:
+      if thefile.units[0].isheader():
+        thenewfile.units.insert(0, thefile.units[0])
+      else:
+        thenewfile.units.insert(0, thenewfile.makeheader())
+    return thenewfile
 
 class GrepOptionParser(optrecurse.RecursiveOptionParser):
   """a specialized Option Parser for the grep tool..."""
@@ -144,25 +151,25 @@ class GrepOptionParser(optrecurse.RecursiveOptionParser):
     (options, args) = self.parse_args()
     options.inputformats = self.inputformats
     options.outputoptions = self.outputoptions
-    options.checkfilter = pogrepfilter(options.searchstring, options.searchparts, options.ignorecase, options.useregexp, options.invertmatch, options.accelchar, locale.getpreferredencoding())
+    options.checkfilter = GrepFilter(options.searchstring, options.searchparts, options.ignorecase, options.useregexp, options.invertmatch, options.accelchar, locale.getpreferredencoding(), options.includeheader)
     self.usepsyco(options)
     self.recursiveprocess(options)
 
-def runpogrep(inputfile, outputfile, templatefile, checkfilter):
-  """reads in inputfile using po.pofile, filters using pocheckfilter, writes to stdout"""
-  fromfile = po.pofile(inputfile)
+def rungrep(inputfile, outputfile, templatefile, checkfilter):
+  """reads in inputfile, filters using checkfilter, writes to outputfile"""
+  fromfile = factory.getobject(inputfile)
   tofile = checkfilter.filterfile(fromfile)
   if tofile.isempty():
-    return 0
+    return False
   outputfile.write(str(tofile))
-  return 1
+  return True
 
 def cmdlineparser():
-  formats = {"po":("po", runpogrep), "pot":("pot", runpogrep), None:("po", runpogrep)}
+  formats = {"po":("po", rungrep), "pot":("pot", rungrep), "xliff":("xliff", rungrep), "xlf":("xlf", rungrep), "xlff":("xlff", rungrep), None:("po", rungrep)}
   parser = GrepOptionParser(formats)
   parser.add_option("", "--search", dest="searchparts",
-    action="append", type="choice", choices=["msgid", "msgstr", "comment", "source"],
-    metavar="SEARCHPARTS", help="searches the given parts (msgid, msgstr, comment, source)")
+    action="append", type="choice", choices=["source", "target", "notes", "locations", "msgid", "msgstr", "comment" ],
+    metavar="SEARCHPARTS", help="searches the given parts (source, target, notes and locations)")
   parser.add_option("-I", "--ignore-case", dest="ignorecase",
     action="store_true", default=False, help="ignore case distinctions")
   parser.add_option("-e", "--regexp", dest="useregexp",
@@ -172,10 +179,17 @@ def cmdlineparser():
   parser.add_option("", "--accelerator", dest="accelchar",
     action="store", type="choice", choices=["&", "_", "~"],
     metavar="ACCELERATOR", help="ignores the given accelerator when matching")
+  parser.add_option("", "--header", dest="includeheader",
+    action="store_true", default=False,
+    help="include a PO header in the output")
   parser.set_usage()
   parser.passthrough.append('checkfilter')
+  parser.description=__doc__
   return parser
 
 def main():
   parser = cmdlineparser()
   parser.run()
+
+if __name__ == '__main__':
+    main()
